import reflex as rx

import reflex_enterprise as rxe

from .common import demo


@rx.memo
def memo_combobox():
    return rxe.mantine.combobox(
        rxe.mantine.combobox.target(
            rx.input(
                type="button",
                on_click=lambda: rx.Var("combobox.toggleDropdown()").to(rx.EventChain),
            ),
        ),
        rxe.mantine.combobox.dropdown(
            rxe.mantine.combobox.options(
                rxe.mantine.combobox.option("Option 1"),
                rxe.mantine.combobox.option("Option 2"),
                rxe.mantine.combobox.option("Option 3"),
            ),
        ),
        label="Combobox",
        placeholder="Select a value",
        # on_change=rx.toast("Selected value: {value}"),
    )


@demo(
    route="/combobox",
    title="Combobox",
    description="A simple combobox example using Mantine.",
)
def combobox_page():
    return memo_combobox()



"""Timeline component for displaying content divided into sections."""

from typing import Literal

import reflex as rx
from reflex.components.component import Component

from reflex_enterprise.components.mantine.base import MantineCoreBase


class Timeline(MantineCoreBase):
    """Display content divided into sections."""

    tag = "Timeline"

    # Index of the active item, -1 by default
    active: rx.Var[int]

    # Timeline alignment
    align: rx.Var[Literal["right", "left"]]

    # Determines whether icon color should depend on background-color. If luminosity of the color prop is less than theme.luminosityThreshold, then theme.white will be used for text color, otherwise theme.black. Overrides theme.autoContrast.
    auto_contrast: rx.Var[bool]

    # Bullet size
    bullet_size: rx.Var[float | str]

    # Active color from theme.colors
    color: rx.Var[str]

    # Line width
    line_width: rx.Var[float | str]

    # Radius from theme.radius, or number to calculate border-radius in px
    radius: rx.Var[float | str]

    # Reverse active direction without changing items order
    reverse_active: rx.Var[bool]

    _valid_children = ["TimelineItem"]


class TimelineItem(MantineCoreBase):
    """Item component for Timeline."""

    tag = "Timeline.Item"

    # Custom bullet replacing the default one
    bullet: rx.Var[Component | str]

    # Active color from theme.colors
    color: rx.Var[str]

    # Line border style
    line_variant: rx.Var[Literal["solid", "dashed", "dotted"]]

    # Radius from theme.radius, or number to calculate border-radius in px
    radius: rx.Var[float | str]

    # Item title, rendered next to the bullet
    title: rx.Var[str | Component]


# Define the namespace
class TimelineNamespace(rx.ComponentNamespace):
    """Timeline components namespace."""

    __call__ = Timeline.create
    item = TimelineItem.create


timeline = TimelineNamespace()



"""Utility functions for AG Grid components."""

from typing import Any

import reflex as rx
from reflex.vars.base import Var

from .resources.base import AgGridParamsBase


def format_parameter_prop(
    props: dict[str, Any],
    param_class: type[AgGridParamsBase],
) -> None:
    """Format parameter props using class naming conventions.

    Args:
        props: The props dictionary to modify
        param_class: The parameter class with auto-naming methods
    """
    prop_name = param_class.get_prop_name()
    formatter_function_name = param_class.get_formatter_function_name()

    if (param_value := props.get(prop_name)) is not None:
        if isinstance(param_value, dict):
            # Static dict: process with parameter class
            props[prop_name] = param_class(**param_value).dict()
        elif isinstance(param_value, Var):
            # Var: call JavaScript formatter at runtime
            props[prop_name] = rx.vars.function.FunctionStringVar.create(
                formatter_function_name
            ).call(param_value)



"""Stub file for reflex_enterprise/components/map/map_consumer.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Mapping, Optional, Sequence

from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from reflex_enterprise.components.map.base import LazyBaseLeafletComponent

from .types import (
    ErrorEvent,
    LatLng,
    LayerEvent,
    MouseEvent,
    MoveEvent,
    PopupEvent,
    ResizeEvent,
    TooltipEvent,
    ZoomEvent,
    ZoomLevelsChangeEvent,
)

class MapConsumer(LazyBaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        map_ref: Var[str] | str | None = None,
        events_handler: Var[dict] | dict | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_contextmenu: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_dblclick: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_layeradd: Optional[EventType[()] | EventType[LayerEvent]] = None,
        on_layerremove: Optional[EventType[()] | EventType[LayerEvent]] = None,
        on_load: Optional[EventType[()] | EventType[str]] = None,
        on_locationerror: Optional[EventType[()] | EventType[ErrorEvent]] = None,
        on_locationfound: Optional[EventType[()] | EventType[LatLng]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_mousedown: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_mousemove: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_mouseout: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_mouseover: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_mouseup: Optional[EventType[()] | EventType[MouseEvent]] = None,
        on_move: Optional[EventType[()] | EventType[MoveEvent]] = None,
        on_move_end: Optional[EventType[()] | EventType[MoveEvent]] = None,
        on_move_start: Optional[EventType[()] | EventType[MoveEvent]] = None,
        on_popupclose: Optional[EventType[()] | EventType[PopupEvent]] = None,
        on_popupopen: Optional[EventType[()] | EventType[PopupEvent]] = None,
        on_resize: Optional[EventType[()] | EventType[ResizeEvent]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_tooltipclose: Optional[EventType[()] | EventType[TooltipEvent]] = None,
        on_tooltipopen: Optional[EventType[()] | EventType[TooltipEvent]] = None,
        on_unload: Optional[EventType[()] | EventType[str]] = None,
        on_unmount: Optional[EventType[()]] = None,
        on_viewreset: Optional[EventType[()] | EventType[str]] = None,
        on_zoom: Optional[EventType[()] | EventType[ZoomEvent]] = None,
        on_zoom_end: Optional[EventType[()] | EventType[ZoomEvent]] = None,
        on_zoom_levels_change: Optional[
            EventType[()] | EventType[ZoomLevelsChangeEvent]
        ] = None,
        on_zoom_start: Optional[EventType[()] | EventType[ZoomEvent]] = None,
        **props,
    ) -> "MapConsumer":
        """Create a MapConsumer.

        This method maps Python-style event handlers (on_move) to
        the Leaflet-style event names (move).
        """
        ...

    def add_hooks(self) -> list[Var]: ...



"""Reusable config mixin for OIDC providers."""

import os
from typing import ClassVar

from reflex_enterprise.auth.oidc.types import AsyncHTTPClientProtocol
from reflex_enterprise.auth.oidc.utils import DEFAULT_HTTP_CLIENT, issuer_endpoint


class ConfigMixin:
    """Customizable per-provider configuration from env var helper."""

    __provider__: ClassVar[str] = "base"

    @classmethod
    def _http_client(cls) -> AsyncHTTPClientProtocol:
        """Get the HTTP client instance for OIDC operations."""
        return DEFAULT_HTTP_CLIENT

    async def _get_config_value(
        self, key: str, default: str | None = None
    ) -> str | None:
        """Get a configuration value for the OIDC provider.

        Args:
            key: The configuration key to retrieve.
            default: The default value if the key is not found.

        Returns:
            The configuration value as a string, or the default if not found.
        """
        full_key = f"{self.__provider__.upper()}_{key.upper()}"
        fallback_key = f"OIDC_{key.upper()}"
        return os.environ.get(full_key, os.environ.get(fallback_key, default))

    @classmethod
    def _with_provider_prefix(cls, url: str) -> str:
        """Prefix a URL path with the provider name for namespacing.

        Args:
            url: The URL path to prefix.

        Returns:
            The namespaced URL path.
        """
        prefix = f"/_reflex_oidc_{cls.__provider__}"
        if not url.startswith("/"):
            url = "/" + url
        return prefix + url

    # Informational helper methods that should be overwritten by subclasses as needed
    @classmethod
    def _authorization_code_endpoint(cls) -> str:
        """Get the authorization code endpoint path."""
        return cls._with_provider_prefix("/authorization-code/callback")

    @classmethod
    def _popup_login_endpoint(cls) -> str:
        """Get the popup login endpoint path."""
        return cls._with_provider_prefix("/popup-login")

    @classmethod
    def _popup_logout_endpoint(cls) -> str:
        """Get the popup logout endpoint path."""
        return cls._with_provider_prefix("/popup-logout")

    async def _client_id(self) -> str:
        """Get the OIDC client ID."""
        client_id = await self._get_config_value("client_id")
        if not client_id:
            raise RuntimeError("OIDC client ID not configured")
        return client_id

    async def _client_secret(self) -> str:
        """Get the OIDC client secret."""
        return await self._get_config_value("client_secret") or ""

    async def _issuer_uri(self) -> str:
        """Get the OIDC issuer URI."""
        issuer = await self._get_config_value("issuer_uri")
        if not issuer:
            raise RuntimeError("OIDC issuer URI not configured")
        return issuer

    async def _issuer_endpoint(self, service: str) -> str:
        """Get an endpoint URL (authorization/token/userinfo/etc) from OIDC metadata.

        Args:
            service: The OIDC service endpoint to retrieve.

        Returns:
            The endpoint URL as a string.
        """
        return await issuer_endpoint(
            await self._issuer_uri(), service, client=self._http_client()
        )



"""Demonstrates the use of aligned grids in Reflex."""

import pandas as pd
import reflex as rx

import reflex_enterprise as rxe

from .common import demo


class AlignedState(rx.State):
    """State for the aligned grids demo."""

    data: list[dict] = []

    @rx.event
    def load_data(self):
        """Load data into the state."""
        df = pd.read_json("https://www.ag-grid.com/example-assets/olympic-winners.json")
        self.data = df.to_dict("records")


column_defs = [
    {"field": "athlete"},
    {"field": "age"},
    {"field": "country"},
    {"field": "year"},
    {"field": "sport"},
    {
        "header_name": "Medals",
        "children": [
            rxe.ag_grid.column_def(
                field="total",
                column_group_show="closed",
                col_id="total",
                value_getter="params.data.gold + params.data.silver + params.data.bronze",
                width=100,
            ),
            {"field": "gold", "column_group_show": "open", "width": 100},
            {"field": "silver", "column_group_show": "open", "width": 100},
            {"field": "bronze", "column_group_show": "open", "width": 100},
        ],
    },
]


@demo(
    route="/aligned-grids",
    title="Aligned Grids",
    description="Demonstrates the use of aligned grids in Reflex.",
    on_load=AlignedState.load_data,
)
def aligned_grids_page():
    """Aligned grids demo."""
    return rxe.ag_grid(
        id="grid1",
        column_defs=column_defs,
        row_data=AlignedState.data,
        aligned_grids=["grid2"],
        width="100%",
    ), rxe.ag_grid(
        id="grid2",
        column_defs=column_defs,
        row_data=AlignedState.data,
        aligned_grids=["grid1"],
        width="100%",
    )



"""Stub file for reflex_enterprise/components/mantine/pills_input.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Literal, Mapping, Optional, Sequence

import reflex as rx
from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import MantineCoreBase

LiteralPillsInputSize = Literal["xs", "sm", "md", "lg", "xl"]
LiteralPillsInputFieldType = Literal["input", "pills", "cursor"]

class PillsInput(MantineCoreBase):
    @classmethod
    def create(
        cls,
        *children,
        size: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"]]
        | None = None,
        disabled: Var[bool] | bool | None = None,
        error: Var[str] | str | None = None,
        variant: Var[str] | str | None = None,
        field_ref: Any | Var[Any] | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "PillsInput":
        """Create the Reflex Enterprise component."""
        ...

class PillsInputField(MantineCoreBase):
    @classmethod
    def create(
        cls,
        *children,
        type: Literal["cursor", "input", "pills"]
        | Var[Literal["cursor", "input", "pills"]]
        | None = None,
        placeholder: Var[str] | str | None = None,
        disabled: Var[bool] | bool | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "PillsInputField":
        """Create the Reflex Enterprise component."""
        ...

class PillsInputNamespace(rx.ComponentNamespace):
    field = staticmethod(PillsInputField.create)

    @staticmethod
    def __call__(
        *children,
        size: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"]]
        | None = None,
        disabled: Var[bool] | bool | None = None,
        error: Var[str] | str | None = None,
        variant: Var[str] | str | None = None,
        field_ref: Any | Var[Any] | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "PillsInput":
        """Create the Reflex Enterprise component."""
        ...

pills_input = PillsInputNamespace()



"""Hooks for the flow component."""

from typing import TYPE_CHECKING, Any, Mapping, Sequence, TypedDict

import reflex

from .flow import LIBRARY
from .types import (
    ConnectionState,
    ConnectionStateVar,
    Edge,
    EdgeVar,
    HandleType,
    Id,
    InternalNode,
    InternalNodeVar,
    Node,
    NodeConnection,
    NodeVar,
    Rect,
    RectVar,
    Viewport,
    ViewportVar,
    XYPosition,
    XYPositionVar,
)

react_flow_instance = reflex.Var(
    "reactFlowInstance",
    _var_data=reflex.vars.VarData(
        hooks=["const reactFlowInstance = useReactFlow();"],
        imports={LIBRARY: ["useReactFlow"]},
    ),
).to(dict)


# TODO: wrap deleteElements, getHandleConnections, getNodeConnections
# TODO: wrap isNodeIntersecting
# TODO: wrap zoomIn, zoomOut, zoomTo, getZoom, setViewport, getViewport
# TODO: wrap setCenter, fitBounds, viewportInitialized, fitView


def get_nodes() -> reflex.vars.ArrayVar[list[Node]]:
    """Returns an array of all nodes in the flow."""
    return (
        react_flow_instance["getNodes"]
        .to(reflex.vars.FunctionVar)
        .call()
        .to(list[Node])
    )  # pyright: ignore[reportReturnType]


def set_nodes(
    nodes: Sequence[Node] | reflex.Var[Sequence[Node]],
) -> reflex.vars.base.NoneVar:
    """Sets the nodes in the flow."""
    return react_flow_instance["setNodes"].to(reflex.vars.FunctionVar).call(nodes)  # pyright: ignore[reportReturnType]


def add_nodes(
    nodes: Node | Sequence[Node] | reflex.Var[Node | Sequence[Node]],
) -> reflex.vars.base.NoneVar:
    """Adds nodes to the flow."""
    return react_flow_instance["addNodes"].to(reflex.vars.FunctionVar).call(nodes)  # pyright: ignore[reportReturnType]


def get_node(
    id: str | reflex.Var[str],
) -> NodeVar | reflex.vars.base.NoneVar:
    """Returns a node by its ID."""
    return react_flow_instance["getNode"].to(reflex.vars.FunctionVar).call(id)  # pyright: ignore[reportReturnType]


def get_internal_node(
    id: str | reflex.Var[str],
) -> InternalNodeVar | reflex.vars.base.NoneVar:
    """Returns an internal node by its ID."""
    return (
        react_flow_instance["getInternalNode"]
        .to(reflex.vars.FunctionVar)
        .call(id)
        .to(InternalNode | None)
    )  # pyright: ignore[reportReturnType]


def get_edges() -> reflex.vars.ArrayVar[list[Edge]]:
    """Returns an array of all edges in the flow."""
    return (
        react_flow_instance["getEdges"]
        .to(reflex.vars.FunctionVar)
        .call()
        .to(list[Edge])
    )


def set_edges(
    edges: Sequence[Edge] | reflex.Var[Sequence[Edge]],
) -> reflex.vars.base.NoneVar:
    """Sets the edges in the flow."""
    return react_flow_instance["setEdges"].to(reflex.vars.FunctionVar).call(edges)  # pyright: ignore[reportReturnType]


def add_edges(
    edges: Edge | Sequence[Edge] | reflex.Var[Edge | Sequence[Edge]],
) -> reflex.vars.base.NoneVar:
    """Adds edges to the flow."""
    return react_flow_instance["addEdges"].to(reflex.vars.FunctionVar).call(edges)  # pyright: ignore[reportReturnType]


def get_edge(
    id: str | reflex.Var[str],
) -> EdgeVar | reflex.vars.base.NoneVar:
    """Returns an edge by its ID."""
    return (
        react_flow_instance["getEdge"]
        .to(reflex.vars.FunctionVar)
        .call(id)
        .to(Edge | None)
    )  # pyright: ignore[reportReturnType]


class ReactFlowJsonObject(TypedDict):
    """Represents the JSON object for the React Flow state."""

    nodes: list[Node]
    edges: list[Edge]
    viewport: Viewport


class ReactFlowJsonObjectVar(reflex.vars.ObjectVar[ReactFlowJsonObject]):
    """Var for ReactFlowJsonObject."""

    if TYPE_CHECKING:
        nodes: reflex.vars.ArrayVar[list[Node]]
        edges: reflex.vars.ArrayVar[list[Edge]]
        viewport: ViewportVar


def to_object() -> ReactFlowJsonObjectVar:
    """Converts the React Flow state to a JSON object."""
    return react_flow_instance["toObject"].to(reflex.vars.FunctionVar).call()  # pyright: ignore[reportReturnType]


def update_node(
    id: str | reflex.Var[str],
    node_update: Node | reflex.Var[Node],
    *,
    replace: bool | reflex.Var[bool] | None = None,
):
    """Updates a node in the flow."""
    options = (
        {
            "replace": replace,
        }
        if replace is not None
        else {}
    )
    return (
        react_flow_instance["updateNode"]
        .to(reflex.vars.FunctionVar)
        .call(id, node_update, options)
    )  # pyright: ignore[reportReturnType]


def update_node_data(
    id: str | reflex.Var[str],
    data_update: Mapping[str, Any] | reflex.Var[Mapping[str, Any]],
    *,
    replace: bool | reflex.Var[bool] | None = None,
) -> reflex.vars.base.NoneVar:
    """Updates a node's data in the flow."""
    options = (
        {
            "replace": replace,
        }
        if replace is not None
        else {}
    )
    return (
        react_flow_instance["updateNodeData"]
        .to(reflex.vars.FunctionVar)
        .call(id, data_update, options)
    )  # pyright: ignore[reportReturnType]


def update_edge(
    id: str | reflex.Var[str],
    edge_update: Edge | reflex.Var[Edge],
    *,
    replace: bool | reflex.Var[bool] | None = None,
) -> reflex.vars.base.NoneVar:
    """Updates an edge in the flow."""
    options = (
        {
            "replace": replace,
        }
        if replace is not None
        else {}
    )
    return (
        react_flow_instance["updateEdge"]
        .to(reflex.vars.FunctionVar)
        .call(id, edge_update, options)
    )  # pyright: ignore[reportReturnType]


def update_edge_data(
    id: str | reflex.Var[str],
    data_update: Mapping[str, Any] | reflex.Var[Mapping[str, Any]],
    *,
    replace: bool | reflex.Var[bool] | None = None,
) -> reflex.vars.base.NoneVar:
    """Updates an edge's data in the flow."""
    options = (
        {
            "replace": replace,
        }
        if replace is not None
        else {}
    )
    return (
        react_flow_instance["updateEdgeData"]
        .to(reflex.vars.FunctionVar)
        .call(id, data_update, options)
    )  # pyright: ignore[reportReturnType]


def get_nodes_bounds(
    nodes: Sequence[str | Node] | reflex.Var[Sequence[str | Node]],
) -> RectVar:
    """Returns the bounds of the given nodes."""
    return (
        react_flow_instance["getNodesBounds"]
        .to(reflex.vars.FunctionVar)
        .call(nodes)
        .to(Rect)
    )  # pyright: ignore[reportReturnType]


def screen_to_flow_position(
    *,
    x: float | reflex.Var[float],
    y: float | reflex.Var[float],
    snap_to_grid: bool | reflex.Var[bool] | None = None,
) -> XYPositionVar:
    """With this function you can translate a screen pixel position to a flow position. It is useful for implementing drag and drop from a sidebar for example."""
    options = {"snapToGrid": snap_to_grid} if snap_to_grid is not None else {}
    return (
        react_flow_instance["screenToFlowPosition"]
        .to(reflex.vars.FunctionVar)
        .call({"x": x, "y": y}, options)
    ).to(XYPosition)  # pyright: ignore[reportReturnType]


def flow_to_screen_position(
    *,
    x: float | reflex.Var[float],
    y: float | reflex.Var[float],
) -> XYPositionVar:
    """Translate a position inside the flow’s canvas to a screen pixel position."""
    return (
        react_flow_instance["flowToScreenPosition"]
        .to(reflex.vars.FunctionVar)
        .call({"x": x, "y": y})
    ).to(XYPosition)  # pyright: ignore[reportReturnType]


# This doesn't wrap getNodeConnections because that one is weird
def get_node_connections(
    *,
    id: str | reflex.Var[str] | None = None,
    handle_type: HandleType | reflex.Var[HandleType] | None = None,
    handle_id: str | reflex.Var[str | None] | None = None,
) -> reflex.vars.ArrayVar[list[NodeConnection]]:
    """This hook returns an array of connections on a specific node, handle type ("source", "target") or handle ID."""
    options = reflex.Var.create(
        {
            "id": id,
            "handleType": handle_type,
            "handleId": handle_id,
        }
    )
    id = reflex.vars.get_unique_variable_name()
    return (
        reflex.Var(
            f"connections_{id}",
            _var_data=reflex.vars.VarData.merge(
                options._get_all_var_data(),
                reflex.vars.VarData(
                    hooks=[
                        f"const connections_{id} = useNodeConnections({options!s});"
                    ],
                    imports={
                        LIBRARY: ["useNodeConnections"],
                    },
                ),
            ),
        )
    ).to(list[NodeConnection])


def get_connection() -> ConnectionStateVar:
    """The useConnection hook returns the current connection state when there is an active connection interaction.

    If no connection interaction is active, it returns null for every property.

    A typical use case for this hook is to colorize handles based on a certain condition (e.g. if the connection is valid or not).
    """
    id = reflex.vars.get_unique_variable_name()
    return (
        reflex.Var(
            f"connection_{id}",
            _var_data=reflex.vars.VarData(
                hooks=[f"const connection_{id} = useConnection();"],
                imports={
                    LIBRARY: ["useConnection"],
                },
            ),
        )
    ).to(ConnectionState)  # pyright: ignore[reportReturnType]


def get_intersecting_nodes(
    node: Node | Rect | Id | reflex.Var[Node | Rect | Id],
    *,
    partially: bool | reflex.Var[bool] = True,
    nodes: Sequence[Node | Rect | Id]
    | reflex.Var[Sequence[Node | Rect | Id]]
    | None = None,
) -> reflex.vars.ArrayVar[list[Node]]:
    """Find all the nodes currently intersecting with a given node or rectangle.

    The partially parameter can be set to true to include nodes that are only partially intersecting.
    """
    return (
        react_flow_instance["getIntersectingNodes"]
        .to(reflex.vars.FunctionVar)
        .call(node, partially, nodes)
        .to(list[Node])
    )



"""Map demo showcasing different map controls."""

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.map.types import latlng

from .common import demo


class ControlsState(rx.State):
    """State class for the map controls demo."""

    pass


@demo(
    route="/map-controls",
    title="Map Controls",
    description="Demonstrates different map controls including zoom, scale, attribution, and layers.",
)
def map_controls() -> rx.Component:
    # Create coordinates for map center
    london = latlng(lat=51.505, lng=-0.09)
    map_id = "map-controls-demo"

    return rx.vstack(
        rx.heading("Map Controls Demo"),
        rx.text("This map demonstrates different controls available in Leaflet."),
        rxe.map(
            # Basic tile layer
            rxe.map.tile_layer(
                url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
            ),
            # Layers control (commented out as it doesn't work as expected yet)
            # rxe.map.layers_control(
            #     position="topright",
            #     collapsed=False,
            # ),
            # Note: The following code is commented out as it doesn't work as expected yet
            # LayersControl with BaseLayer and Overlay will be implemented in a future update
            # rxe.map.layers_control(
            #     rxe.map.layers_control_base_layer(
            #         rxe.map.tile_layer(
            #             url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
            #         ),
            #         name="OpenStreetMap",
            #         checked=True,
            #     ),
            #     rxe.map.layers_control_base_layer(
            #         rxe.map.tile_layer(
            #             url="https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}",
            #         ),
            #         name="Satellite",
            #     ),
            #     # Overlay layers
            #     rxe.map.layers_control_overlay(
            #         rxe.map.circle(
            #             center=london,
            #             radius=1000,  # 1000 meters
            #             path_options=rxe.map.path_options(
            #                 color="#ff0000",
            #                 fill_color="#ff3333",
            #                 fill_opacity=0.5,
            #             ),
            #         ),
            #         name="Circle (1km)",
            #         checked=True,
            #     ),
            #     rxe.map.layers_control_overlay(
            #         rxe.map.marker(
            #             rxe.map.tooltip("London"),
            #             position=london,
            #         ),
            #         name="Marker",
            #         checked=True,
            #     ),
            #     position="topright",
            #     collapsed=False,
            # ),
            # Add ZoomControl
            rxe.map.zoom_control(
                position="topright",
            ),
            # # Add ScaleControl
            rxe.map.scale_control(
                position="bottomleft",
            ),
            # # Add AttributionControl
            rxe.map.attribution_control(
                position="topleft",
            ),
            # Map settings
            id=map_id,
            zoom=13,
            center=london,
            zoom_control=False,  # Disable default zoom control
            attribution_control=False,  # Disable default attribution control
            width="100%",
            height="50vh",
        ),
        rx.vstack(
            rx.text("Available Controls:"),
            rx.unordered_list(
                rx.list_item("Zoom Control - Located at topright (default is topleft)"),
                rx.list_item("Scale Control - Located at bottom-left (default is off)"),
                rx.list_item(
                    "Attribution Control - Located at topleft (default is bottom-right)"
                ),
                # rx.list_item(
                #     rx.text(
                #         "Layers Control - Located at top-right (base layer and overlay functionality coming soon)"
                #     )
                # ),
            ),
            align_items="flex-start",
            spacing="2",
            width="100%",
        ),
        width="100%",
        padding="1em",
    )



"""Pill demo page."""

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


@demo(
    route="/pill",
    title="Pill",
    description="A pill is a small, rounded component that can be used to represent a single value or a small set of values.",
)
def pill_page():
    """Pill demo page."""
    return rx.vstack(
        rx.hstack(
            rxe.mantine.pill("Default"),
            rxe.mantine.pill("Default", color="red"),
            rxe.mantine.pill(
                "Default", with_remove_button=True, on_remove=rx.toast("Removed")
            ),
            spacing="2",
        ),
        rxe.mantine.pill.group(
            rxe.mantine.pill("Default", size="xs"),
            rxe.mantine.pill("Default", size="sm"),
            rxe.mantine.pill("Default", size="md"),
            rxe.mantine.pill("Default", size="lg"),
        ),
        spacing="4",
    )



import reflex as rx
from reflex.page import DECORATED_PAGES

import flow.add_nodes_on_edge_drop
import flow.connection_limit
import flow.custom_node
import flow.drag_handle
import flow.intersections
import flow.overview  # noqa: F401
import reflex_enterprise as rxe

app = rxe.App(
    stylesheets=["xy-theme.css", "style.css"],
    theme=rx.theme(appearance="light"),
)

pages = {
    page["title"]: page["route"]
    for _, page in next(DECORATED_PAGES.values().__iter__())
}
app.add_page(
    lambda: rx.vstack(*[rx.link(title, href=route) for title, route in pages.items()]),
    route="/",
)



"""Stub file for reflex_enterprise/components/dnd/dnd.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import (
    Any,
    Callable,
    ClassVar,
    Literal,
    Mapping,
    Optional,
    Sequence,
    TypedDict,
    TypeVar,
)

import reflex as rx
from reflex.components.component import MemoizationLeaf
from reflex.components.core.breakpoints import Breakpoints
from reflex.components.el.elements import Div
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var, VarData

from reflex_enterprise.components.component import ComponentEnterprise
from reflex_enterprise.vars import static

JsObj = Mapping[str, Any] | Var[Mapping[str, Any]] | None
ITEM_T = TypeVar("ITEM_T", bound=JsObj)
COLLECT_PARAMS_T = TypeVar(
    "COLLECT_PARAMS_T", bound=rx.vars.ObjectVar[Mapping[str, Any]]
)
ANY_TYPE = Var.create("Any")
DEP_ARRAY = Var.create(["DEP_ARRAY"])
USE_DRAG = "useDrag"
_use_drag = rx.vars.FunctionStringVar(
    _js_expr=USE_DRAG, _var_data=VarData(imports={"react-dnd": [USE_DRAG]})
)
USE_DROP = "useDrop"
_use_drop = rx.vars.FunctionStringVar(
    _js_expr=USE_DROP, _var_data=VarData(imports={"react-dnd": [USE_DROP]})
)
HTML5_BACKEND = "HTML5Backend"
HTML5Backend = Var(
    HTML5_BACKEND,
    _var_data=VarData(imports={"react-dnd-html5-backend": [HTML5_BACKEND]}),
)
TOUCH_BACKEND = "TouchBackend"
TouchBackend = Var(
    TOUCH_BACKEND,
    _var_data=VarData(imports={"react-dnd-touch-backend": [TOUCH_BACKEND]}),
)

class DnDProvider(ComponentEnterprise):
    @classmethod
    def create(
        cls,
        *children,
        backend: Var[str] | str | None = None,
        context: Var[dict[str, Any]] | dict[str, Any] | None = None,
        options: Var[dict[str, Any]] | dict[str, Any] | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "DnDProvider":
        """Create the Reflex Enterprise component."""
        ...

class BaseDnDDiv(Div, MemoizationLeaf):
    @classmethod
    def create(
        cls,
        *children,
        access_key: Var[str] | str | None = None,
        auto_capitalize: Literal[
            "characters", "none", "off", "on", "sentences", "words"
        ]
        | Var[Literal["characters", "none", "off", "on", "sentences", "words"]]
        | None = None,
        content_editable: Literal["inherit", "plaintext-only"]
        | Var[Literal["inherit", "plaintext-only"] | bool]
        | bool
        | None = None,
        context_menu: Var[str] | str | None = None,
        dir: Var[str] | str | None = None,
        draggable: Var[bool] | bool | None = None,
        enter_key_hint: Literal[
            "done", "enter", "go", "next", "previous", "search", "send"
        ]
        | Var[Literal["done", "enter", "go", "next", "previous", "search", "send"]]
        | None = None,
        hidden: Var[bool] | bool | None = None,
        input_mode: Literal[
            "decimal", "email", "none", "numeric", "search", "tel", "text", "url"
        ]
        | Var[
            Literal[
                "decimal", "email", "none", "numeric", "search", "tel", "text", "url"
            ]
        ]
        | None = None,
        item_prop: Var[str] | str | None = None,
        lang: Var[str] | str | None = None,
        role: Literal[
            "alert",
            "alertdialog",
            "application",
            "article",
            "banner",
            "button",
            "cell",
            "checkbox",
            "columnheader",
            "combobox",
            "complementary",
            "contentinfo",
            "definition",
            "dialog",
            "directory",
            "document",
            "feed",
            "figure",
            "form",
            "grid",
            "gridcell",
            "group",
            "heading",
            "img",
            "link",
            "list",
            "listbox",
            "listitem",
            "log",
            "main",
            "marquee",
            "math",
            "menu",
            "menubar",
            "menuitem",
            "menuitemcheckbox",
            "menuitemradio",
            "navigation",
            "none",
            "note",
            "option",
            "presentation",
            "progressbar",
            "radio",
            "radiogroup",
            "region",
            "row",
            "rowgroup",
            "rowheader",
            "scrollbar",
            "search",
            "searchbox",
            "separator",
            "slider",
            "spinbutton",
            "status",
            "switch",
            "tab",
            "table",
            "tablist",
            "tabpanel",
            "term",
            "textbox",
            "timer",
            "toolbar",
            "tooltip",
            "tree",
            "treegrid",
            "treeitem",
        ]
        | Var[
            Literal[
                "alert",
                "alertdialog",
                "application",
                "article",
                "banner",
                "button",
                "cell",
                "checkbox",
                "columnheader",
                "combobox",
                "complementary",
                "contentinfo",
                "definition",
                "dialog",
                "directory",
                "document",
                "feed",
                "figure",
                "form",
                "grid",
                "gridcell",
                "group",
                "heading",
                "img",
                "link",
                "list",
                "listbox",
                "listitem",
                "log",
                "main",
                "marquee",
                "math",
                "menu",
                "menubar",
                "menuitem",
                "menuitemcheckbox",
                "menuitemradio",
                "navigation",
                "none",
                "note",
                "option",
                "presentation",
                "progressbar",
                "radio",
                "radiogroup",
                "region",
                "row",
                "rowgroup",
                "rowheader",
                "scrollbar",
                "search",
                "searchbox",
                "separator",
                "slider",
                "spinbutton",
                "status",
                "switch",
                "tab",
                "table",
                "tablist",
                "tabpanel",
                "term",
                "textbox",
                "timer",
                "toolbar",
                "tooltip",
                "tree",
                "treegrid",
                "treeitem",
            ]
        ]
        | None = None,
        slot: Var[str] | str | None = None,
        spell_check: Var[bool] | bool | None = None,
        tab_index: Var[int] | int | None = None,
        title: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "BaseDnDDiv":
        """Create a new memoization leaf component.

        Args:
            *children: The children of the component.
            access_key: Provides a hint for generating a keyboard shortcut for the current element.
            auto_capitalize: Controls whether and how text input is automatically capitalized as it is entered/edited by the user.
            content_editable: Indicates whether the element's content is editable.
            context_menu: Defines the ID of a <menu> element which will serve as the element's context menu.
            dir: Defines the text direction. Allowed values are ltr (Left-To-Right) or rtl (Right-To-Left)
            draggable: Defines whether the element can be dragged.
            enter_key_hint: Hints what media types the media element is able to play.
            hidden: Defines whether the element is hidden.
            input_mode: Defines the type of the element.
            item_prop: Defines the name of the element for metadata purposes.
            lang: Defines the language used in the element.
            role: Defines the role of the element.
            slot: Assigns a slot in a shadow DOM shadow tree to an element.
            spell_check: Defines whether the element may be checked for spelling errors.
            tab_index: Defines the position of the current element in the tabbing order.
            title: Defines a tooltip for the element.
            style: The style of the component.
            key: A unique key for the component.
            id: The id for the component.
            ref: The Var to pass as the ref to the component.
            class_name: The class name for the component.
            custom_attrs: custom attribute
            **props: The props of the component.

        Returns:
            The memoization leaf
        """
        ...

class MonitorBase(Var):
    def get_item_type(self) -> rx.vars.StringVar | Var[None]: ...
    def get_item(self) -> rx.vars.ObjectVar[dict[str, Any]] | Var[None]: ...
    def get_drop_result(self) -> rx.Var: ...
    def did_drop(self) -> rx.vars.BooleanVar: ...
    def get_initial_client_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]: ...
    def get_initial_source_client_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]: ...
    def get_client_offset(self) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]: ...
    def get_difference_from_initial_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]: ...
    def get_source_client_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]: ...

class DragSourceMonitor(MonitorBase):
    def can_drag(self) -> rx.vars.BooleanVar: ...
    def is_dragging(self) -> rx.vars.BooleanVar: ...

class DropTargetMonitor(MonitorBase):
    def can_drop(self) -> rx.vars.BooleanVar: ...
    def is_over(
        self, options: rx.vars.ObjectVar[dict[str, Any]] = Var.create({})
    ) -> rx.vars.BooleanVar: ...

def use_drag(
    type: str | Var[str],
    item: ITEM_T | Callable[[DragSourceMonitor], ITEM_T] = None,
    preview_options: JsObj = None,
    options: JsObj = None,
    end: Callable[[ITEM_T, DragSourceMonitor], None]
    | Var[Callable[[ITEM_T, DragSourceMonitor], None]]
    | None = None,
    can_drag: Callable[[DragSourceMonitor], bool | Var[bool]]
    | Var[Callable[[DragSourceMonitor], bool | Var[bool]]]
    | None = None,
    is_dragging: Callable[[DragSourceMonitor], bool | Var[bool]]
    | Var[Callable[[DragSourceMonitor], bool | Var[bool]]]
    | None = None,
    collect: Callable[[DragSourceMonitor], COLLECT_PARAMS_T]
    | Var[Callable[[DragSourceMonitor], COLLECT_PARAMS_T]]
    | None = None,
    _collected_params: Var[COLLECT_PARAMS_T] | None = None,
    _drag_ref: Var | None = None,
    _drag_preview_ref: Var | None = None,
) -> tuple[COLLECT_PARAMS_T, rx.Var, rx.Var]: ...

class Draggable(BaseDnDDiv):
    @classmethod
    def create(
        cls,
        *children,
        type: Var[str] | str | None = None,
        item: Callable[
            [DragSourceMonitor],
            Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
        ]
        | Mapping[str, Any]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Var[
            Callable[
                [DragSourceMonitor],
                Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
            ]
            | Mapping[str, Any]
            | Mapping[str, Any]
            | Var[Mapping[str, Any]]
            | None
        ]
        | None = None,
        preview_options: Mapping[str, Any]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Var[Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None]
        | None = None,
        options: Mapping[str, Any]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Var[Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None]
        | None = None,
        drag_ref: Any | Var[Any] | None = None,
        drag_preview_ref: Any | Var[Any] | None = None,
        end: Callable[
            [
                Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
                DragSourceMonitor,
            ]
        ]
        | Var[
            Callable[
                [
                    Mapping[str, Any]
                    | Mapping[str, Any]
                    | Var[Mapping[str, Any]]
                    | None,
                    DragSourceMonitor,
                ]
            ]
        ]
        | None = None,
        can_drag: Callable[[DragSourceMonitor], Var[bool] | bool | bool]
        | Var[Callable[[DragSourceMonitor], Var[bool] | bool | bool]]
        | None = None,
        is_dragging: Callable[[DragSourceMonitor], Var[bool] | bool | bool]
        | Var[Callable[[DragSourceMonitor], Var[bool] | bool | bool]]
        | None = None,
        collect: Callable[
            [DragSourceMonitor],
            Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
        ]
        | Var[
            Callable[
                [DragSourceMonitor],
                Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
            ]
        ]
        | None = None,
        access_key: Var[str] | str | None = None,
        auto_capitalize: Literal[
            "characters", "none", "off", "on", "sentences", "words"
        ]
        | Var[Literal["characters", "none", "off", "on", "sentences", "words"]]
        | None = None,
        content_editable: Literal["inherit", "plaintext-only"]
        | Var[Literal["inherit", "plaintext-only"] | bool]
        | bool
        | None = None,
        context_menu: Var[str] | str | None = None,
        dir: Var[str] | str | None = None,
        draggable: Var[bool] | bool | None = None,
        enter_key_hint: Literal[
            "done", "enter", "go", "next", "previous", "search", "send"
        ]
        | Var[Literal["done", "enter", "go", "next", "previous", "search", "send"]]
        | None = None,
        hidden: Var[bool] | bool | None = None,
        input_mode: Literal[
            "decimal", "email", "none", "numeric", "search", "tel", "text", "url"
        ]
        | Var[
            Literal[
                "decimal", "email", "none", "numeric", "search", "tel", "text", "url"
            ]
        ]
        | None = None,
        item_prop: Var[str] | str | None = None,
        lang: Var[str] | str | None = None,
        role: Literal[
            "alert",
            "alertdialog",
            "application",
            "article",
            "banner",
            "button",
            "cell",
            "checkbox",
            "columnheader",
            "combobox",
            "complementary",
            "contentinfo",
            "definition",
            "dialog",
            "directory",
            "document",
            "feed",
            "figure",
            "form",
            "grid",
            "gridcell",
            "group",
            "heading",
            "img",
            "link",
            "list",
            "listbox",
            "listitem",
            "log",
            "main",
            "marquee",
            "math",
            "menu",
            "menubar",
            "menuitem",
            "menuitemcheckbox",
            "menuitemradio",
            "navigation",
            "none",
            "note",
            "option",
            "presentation",
            "progressbar",
            "radio",
            "radiogroup",
            "region",
            "row",
            "rowgroup",
            "rowheader",
            "scrollbar",
            "search",
            "searchbox",
            "separator",
            "slider",
            "spinbutton",
            "status",
            "switch",
            "tab",
            "table",
            "tablist",
            "tabpanel",
            "term",
            "textbox",
            "timer",
            "toolbar",
            "tooltip",
            "tree",
            "treegrid",
            "treeitem",
        ]
        | Var[
            Literal[
                "alert",
                "alertdialog",
                "application",
                "article",
                "banner",
                "button",
                "cell",
                "checkbox",
                "columnheader",
                "combobox",
                "complementary",
                "contentinfo",
                "definition",
                "dialog",
                "directory",
                "document",
                "feed",
                "figure",
                "form",
                "grid",
                "gridcell",
                "group",
                "heading",
                "img",
                "link",
                "list",
                "listbox",
                "listitem",
                "log",
                "main",
                "marquee",
                "math",
                "menu",
                "menubar",
                "menuitem",
                "menuitemcheckbox",
                "menuitemradio",
                "navigation",
                "none",
                "note",
                "option",
                "presentation",
                "progressbar",
                "radio",
                "radiogroup",
                "region",
                "row",
                "rowgroup",
                "rowheader",
                "scrollbar",
                "search",
                "searchbox",
                "separator",
                "slider",
                "spinbutton",
                "status",
                "switch",
                "tab",
                "table",
                "tablist",
                "tabpanel",
                "term",
                "textbox",
                "timer",
                "toolbar",
                "tooltip",
                "tree",
                "treegrid",
                "treeitem",
            ]
        ]
        | None = None,
        slot: Var[str] | str | None = None,
        spell_check: Var[bool] | bool | None = None,
        tab_index: Var[int] | int | None = None,
        title: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_end: Optional[EventType[()] | EventType[dict[str, Any]]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Draggable":
        """Create a Draggable component.

        Will use the `collect_hook` to collect the params if
        not provided, which is represented by the `Draggable.CollectedParams`
        type and accessed via `Draggable.collected_params`.
        """
        ...

    class CollectedParams(TypedDict):
        is_dragging: bool
        can_drag: bool

    @staticmethod
    @static
    def collect_hook(
        monitor: DragSourceMonitor,
    ) -> rx.Var[Draggable.CollectedParams]: ...
    collected_params: ClassVar[rx.vars.ObjectVar[Draggable.CollectedParams]] = (
        rx.vars.ObjectVar("draggableCollectedParams", _var_type=CollectedParams)
    )

def use_drop(
    accept: str | Var[str],
    options: JsObj = None,
    drop: Callable[[dict[str, Any], DropTargetMonitor], None]
    | Var[Callable[[dict[str, Any], DropTargetMonitor], None]]
    | None = None,
    hover: Callable[[dict[str, Any], DropTargetMonitor], None]
    | Var[Callable[[dict[str, Any], DropTargetMonitor], None]]
    | None = None,
    can_drop: Callable[[DropTargetMonitor], bool | Var[bool]]
    | Var[Callable[[DropTargetMonitor], bool | Var[bool]]]
    | None = None,
    collect: Callable[[DropTargetMonitor], COLLECT_PARAMS_T]
    | Var[Callable[[DropTargetMonitor], COLLECT_PARAMS_T]]
    | None = None,
    _collected_params: COLLECT_PARAMS_T | None = None,
    _drop_ref: Var | None = None,
) -> tuple[COLLECT_PARAMS_T, rx.Var]: ...

class DropTarget(BaseDnDDiv):
    @classmethod
    def create(
        cls,
        *children,
        accept: Var[list[str] | str] | list[str] | str | None = None,
        options: Mapping[str, Any]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Var[Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None]
        | None = None,
        drop_ref: Any | Var[Any] | None = None,
        drop: Callable[[dict[str, Any], DropTargetMonitor]]
        | Var[Callable[[dict[str, Any], DropTargetMonitor]]]
        | None = None,
        hover: Callable[[dict[str, Any], DropTargetMonitor]]
        | Var[Callable[[dict[str, Any], DropTargetMonitor]]]
        | None = None,
        can_drop: Callable[[dict[str, Any], DropTargetMonitor], Var[bool] | bool | bool]
        | Var[Callable[[dict[str, Any], DropTargetMonitor], Var[bool] | bool | bool]]
        | None = None,
        collect: Callable[
            [DropTargetMonitor],
            Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
        ]
        | Var[
            Callable[
                [DropTargetMonitor],
                Mapping[str, Any] | Mapping[str, Any] | Var[Mapping[str, Any]] | None,
            ]
        ]
        | None = None,
        access_key: Var[str] | str | None = None,
        auto_capitalize: Literal[
            "characters", "none", "off", "on", "sentences", "words"
        ]
        | Var[Literal["characters", "none", "off", "on", "sentences", "words"]]
        | None = None,
        content_editable: Literal["inherit", "plaintext-only"]
        | Var[Literal["inherit", "plaintext-only"] | bool]
        | bool
        | None = None,
        context_menu: Var[str] | str | None = None,
        dir: Var[str] | str | None = None,
        draggable: Var[bool] | bool | None = None,
        enter_key_hint: Literal[
            "done", "enter", "go", "next", "previous", "search", "send"
        ]
        | Var[Literal["done", "enter", "go", "next", "previous", "search", "send"]]
        | None = None,
        hidden: Var[bool] | bool | None = None,
        input_mode: Literal[
            "decimal", "email", "none", "numeric", "search", "tel", "text", "url"
        ]
        | Var[
            Literal[
                "decimal", "email", "none", "numeric", "search", "tel", "text", "url"
            ]
        ]
        | None = None,
        item_prop: Var[str] | str | None = None,
        lang: Var[str] | str | None = None,
        role: Literal[
            "alert",
            "alertdialog",
            "application",
            "article",
            "banner",
            "button",
            "cell",
            "checkbox",
            "columnheader",
            "combobox",
            "complementary",
            "contentinfo",
            "definition",
            "dialog",
            "directory",
            "document",
            "feed",
            "figure",
            "form",
            "grid",
            "gridcell",
            "group",
            "heading",
            "img",
            "link",
            "list",
            "listbox",
            "listitem",
            "log",
            "main",
            "marquee",
            "math",
            "menu",
            "menubar",
            "menuitem",
            "menuitemcheckbox",
            "menuitemradio",
            "navigation",
            "none",
            "note",
            "option",
            "presentation",
            "progressbar",
            "radio",
            "radiogroup",
            "region",
            "row",
            "rowgroup",
            "rowheader",
            "scrollbar",
            "search",
            "searchbox",
            "separator",
            "slider",
            "spinbutton",
            "status",
            "switch",
            "tab",
            "table",
            "tablist",
            "tabpanel",
            "term",
            "textbox",
            "timer",
            "toolbar",
            "tooltip",
            "tree",
            "treegrid",
            "treeitem",
        ]
        | Var[
            Literal[
                "alert",
                "alertdialog",
                "application",
                "article",
                "banner",
                "button",
                "cell",
                "checkbox",
                "columnheader",
                "combobox",
                "complementary",
                "contentinfo",
                "definition",
                "dialog",
                "directory",
                "document",
                "feed",
                "figure",
                "form",
                "grid",
                "gridcell",
                "group",
                "heading",
                "img",
                "link",
                "list",
                "listbox",
                "listitem",
                "log",
                "main",
                "marquee",
                "math",
                "menu",
                "menubar",
                "menuitem",
                "menuitemcheckbox",
                "menuitemradio",
                "navigation",
                "none",
                "note",
                "option",
                "presentation",
                "progressbar",
                "radio",
                "radiogroup",
                "region",
                "row",
                "rowgroup",
                "rowheader",
                "scrollbar",
                "search",
                "searchbox",
                "separator",
                "slider",
                "spinbutton",
                "status",
                "switch",
                "tab",
                "table",
                "tablist",
                "tabpanel",
                "term",
                "textbox",
                "timer",
                "toolbar",
                "tooltip",
                "tree",
                "treegrid",
                "treeitem",
            ]
        ]
        | None = None,
        slot: Var[str] | str | None = None,
        spell_check: Var[bool] | bool | None = None,
        tab_index: Var[int] | int | None = None,
        title: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_drop: Optional[EventType[()] | EventType[dict[str, Any]]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_hover: Optional[EventType[()] | EventType[dict[str, Any]]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "DropTarget":
        """Create a Droppable component.

        Will use the class `collect_hook` to collect the params if
        not provided, which is represented by the `DropTarget.CollectedParams`
        type and accessed via `DropTarget.collected_params` var.
        """
        ...

    class CollectedParams(TypedDict):
        is_over: bool
        can_drop: bool
        item: dict[str, Any] | None

    @staticmethod
    @static
    def collect_hook(
        monitor: DropTargetMonitor,
    ) -> rx.Var[DropTarget.CollectedParams]: ...
    collected_params: ClassVar[rx.vars.ObjectVar[DropTarget.CollectedParams]] = (
        rx.vars.ObjectVar("dropTargetCollectedParams", _var_type=CollectedParams)
    )

class DnDNamespace(rx.ComponentNamespace):
    HTML5Backend: ...
    TouchBackend: ...
    provider = staticmethod(DnDProvider.create)
    Draggable: type[Draggable]
    draggable = staticmethod(Draggable.create)
    use_drag = staticmethod(use_drag)
    DragSourceMonitor: type[DragSourceMonitor]
    DropTarget: type[DropTarget]
    drop_target = staticmethod(DropTarget.create)
    use_drop = staticmethod(use_drop)
    DropTargetMonitor: type[DropTargetMonitor]

dnd: DnDNamespace



"""Stub file for reflex_enterprise/components/__init__.py"""
# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------

from . import ag_charts, ag_grid, dnd, flow, mantine, map

__all__ = ["ag_charts", "ag_grid", "dnd", "flow", "mantine", "map"]



"""LoadingOverlay component for Mantine."""

from typing import Any, Dict

import reflex as rx

from reflex_enterprise.components.mantine.base import MantineCoreBase


class LoadingOverlay(MantineCoreBase):
    """Display loading overlay over any element."""

    tag = "LoadingOverlay"

    # Indicates whether the overlay is visible.
    visible: rx.Var[bool]

    # Controls overlay z-index.
    z_index: rx.Var[str | int]

    # Props passed down to the Overlay component.
    overlay_props: rx.Var[Dict[str, Any]]

    # Props passed down to the Loader component.
    loader_props: rx.Var[Dict[str, Any]]

    # Props passed down to the Transition component that wraps the Loader, applied when overlay is mounted and unmounted.
    transition_props: rx.Var[Dict[str, Any]]

    @classmethod
    def create(cls, *children: Any, **props: Any) -> rx.Component:
        """Create a LoadingOverlay component."""
        return rx.el.Div.create(
            super().create(**props),
            *children,
            position="relative",
        )


loading_overlay = LoadingOverlay.create



"""The Reflex Enterprise component."""

from reflex.components.component import Component, NoSSRComponent

from reflex_enterprise.app import AppEnterprise


class ComponentEnterprise(Component):
    """The Reflex Enterprise component."""

    @classmethod
    def create(cls, *children, **props) -> Component:
        """Create the Reflex Enterprise component."""
        return super().create(*children, **props)

    def render(self, *children, **props) -> dict:
        """Render the Reflex Enterprise component."""
        from reflex.utils.prerequisites import get_app

        if not isinstance(get_app().app, AppEnterprise):
            raise TypeError(
                "Reflex Enterprise components can only be used in an Enterprise app. Use rxe.App() instead of rx.App()."
            )

        return super().render(*children, **props)


class NoSSRComponentEnterprise(NoSSRComponent):
    """The Reflex Enterprise component without SSR."""

    @classmethod
    def create(cls, *children, **props) -> Component:
        """Create the Reflex Enterprise component."""
        return super().create(*children, **props)

    def render(self, *children, **props) -> dict:
        """Render the Reflex Enterprise component."""
        from reflex.utils.prerequisites import get_app

        if not isinstance(get_app().app, AppEnterprise):
            raise TypeError(
                "Reflex Enterprise components can only be used in an Enterprise app. Use rxe.App() instead of rx.App()."
            )

        return super().render(*children, **props)



import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.flow.types import Edge, Node

initial_nodes: list[Node] = [
    {
        "id": "2",
        "type": "dragHandleNode",
        "dragHandle": ".drag-handle__custom",
        "position": {"x": 200, "y": 200},
    },
]

initial_edges: list[Edge] = []


@rx.memo
def drag_handle_node():
    return rx.fragment(
        rxe.flow.handle(
            type="target",
            position="left",
        ),
        rx.el.div(
            "Only draggable here → ",
            rx.el.span(class_name="drag-handle__custom"),
            class_name="drag-handle__label",
        ),
        rxe.flow.handle(
            type="source",
            position="right",
        ),
    )


@rx.page(route="/nodes/drag-handle", title="Drag Handle Demo")
def drag_handle():
    return rx.box(
        rxe.flow(
            rxe.flow.background(),
            default_nodes=initial_nodes,
            default_edges=initial_edges,
            node_types={"dragHandleNode": drag_handle_node},
            color_mode="light",
            fit_view=True,
        ),
        height="100vh",
        width="100vw",
    )



"""Utility functions for working with flow components."""

from typing import TYPE_CHECKING, Any, Sequence, TypedDict

import reflex

from .flow import LIBRARY
from .types import Connection, Edge, EdgeChange, Node, NodeChange, Position


class Path(TypedDict):
    """Represents an SVG path.

    Args:
        path: the path to use in an SVG <path> element.
        label_x: the x position you can use to render a label for this edge.
        label_y: the y position you can use to render a label for this edge.
        offset_x: the absolute difference between the source x position and the x position of the middle of this path.
        offset_y: the absolute difference between the source y position and the y position of the middle of this path.
    """

    path: str
    label_x: float
    label_y: float
    offset_x: float
    offset_y: float


class PathVar(reflex.vars.ObjectVar[Path]):
    """Represents a SVG path variable."""

    if TYPE_CHECKING:
        path: reflex.vars.StringVar
        label_x: reflex.vars.NumberVar[float]
        label_y: reflex.vars.NumberVar[float]
        offset_x: reflex.vars.NumberVar[float]
        offset_y: reflex.vars.NumberVar[float]


def _array_return_to_object(array: reflex.vars.ArrayVar[Any]):
    return reflex.Var.create(
        {
            "path": array[0],
            "label_x": array[1],
            "label_y": array[2],
            "offset_x": array[3],
            "offset_y": array[4],
        }
    )


def get_simple_bezier_path(
    *,
    source_x: float | reflex.Var[float],
    source_y: float | reflex.Var[float],
    target_x: float | reflex.Var[float],
    target_y: float | reflex.Var[float],
    source_position: Position | reflex.Var[Position] = "bottom",
    target_position: Position | reflex.Var[Position] = "top",
):
    """The getSimpleBezierPath util returns everything you need to render a simple bezier edge between two nodes."""
    unique_id = reflex.vars.get_unique_variable_name()
    variable_name = f"simple_bezier_path_{unique_id}"

    get_simple_bezier_path_call = reflex.vars.FunctionStringVar.create(
        "getSimpleBezierPath"
    ).call(
        {
            "sourceX": source_x,
            "sourceY": source_y,
            "targetX": target_x,
            "targetY": target_y,
            "sourcePosition": source_position,
            "targetPosition": target_position,
        }
    )

    array_name = f"_path_array_{unique_id}"

    result_var = _array_return_to_object(reflex.Var(array_name).to(list))

    return reflex.Var(
        variable_name,
        _var_data=reflex.vars.VarData(
            hooks={
                f"const {array_name} = {get_simple_bezier_path_call!s};": get_simple_bezier_path_call._get_all_var_data(),
                f"const {variable_name} = {result_var!s};": None,
            },
            imports={LIBRARY: ["getSimpleBezierPath"]},
        ),
    ).to(Path)  # pyright: ignore[reportReturnType]


def get_bezier_path(
    *,
    source_x: float | reflex.Var[float],
    source_y: float | reflex.Var[float],
    target_x: float | reflex.Var[float],
    target_y: float | reflex.Var[float],
    source_position: Position | reflex.Var[Position] = "bottom",
    target_position: Position | reflex.Var[Position] = "top",
    curvature: float | reflex.Var[float] = 0.5,
) -> PathVar:
    """Returns everything you need to render a bezier edge between two nodes."""
    unique_id = reflex.vars.get_unique_variable_name()
    variable_name = f"bezier_path_{unique_id}"

    get_bezier_path_call = reflex.vars.FunctionStringVar.create("getBezierPath").call(
        {
            "sourceX": source_x,
            "sourceY": source_y,
            "targetX": target_x,
            "targetY": target_y,
            "sourcePosition": source_position,
            "targetPosition": target_position,
            "curvature": curvature,
        }
    )

    array_name = f"_path_array_{unique_id}"

    result_var = _array_return_to_object(reflex.Var(array_name).to(list))

    return reflex.Var(
        variable_name,
        _var_data=reflex.vars.VarData(
            hooks={
                f"const {array_name} = {get_bezier_path_call!s};": get_bezier_path_call._get_all_var_data(),
                f"const {variable_name} = {result_var!s};": None,
            },
            imports={LIBRARY: ["getBezierPath"]},
        ),
    ).to(Path)  # pyright: ignore[reportReturnType]


def get_straight_path(
    source_x: float | reflex.Var[float],
    source_y: float | reflex.Var[float],
    target_x: float | reflex.Var[float],
    target_y: float | reflex.Var[float],
):
    """Calculates the straight line path between two points."""
    unique_id = reflex.vars.get_unique_variable_name()
    variable_name = f"straight_path_{unique_id}"

    get_straight_path_call = reflex.vars.FunctionStringVar.create(
        "getStraightPath"
    ).call(
        {
            "sourceX": source_x,
            "sourceY": source_y,
            "targetX": target_x,
            "targetY": target_y,
        }
    )

    array_name = f"_path_array_{unique_id}"

    result_var = _array_return_to_object(reflex.Var(array_name).to(list))

    return reflex.Var(
        variable_name,
        _var_data=reflex.vars.VarData(
            hooks={
                f"const {array_name} = {get_straight_path_call!s};": get_straight_path_call._get_all_var_data(),
                f"const {variable_name} = {result_var!s};": None,
            },
            imports={LIBRARY: ["getStraightPath"]},
        ),
    ).to(Path)  # pyright: ignore[reportReturnType]


def get_smooth_step_path(
    *,
    source_x: float | reflex.Var[float],
    source_y: float | reflex.Var[float],
    target_x: float | reflex.Var[float],
    target_y: float | reflex.Var[float],
    source_position: Position | reflex.Var[Position] = "bottom",
    target_position: Position | reflex.Var[Position] = "top",
    border_radius: float | reflex.Var[float] = 5,
    center_x: float | reflex.Var[float] | None = None,
    center_y: float | reflex.Var[float] | None = None,
    offset: float | reflex.Var[float] = 20,
    step_position: float | reflex.Var[float] = 0.5,
) -> PathVar:
    """The getSmoothStepPath util returns everything you need to render a stepped path between two nodes.

    The borderRadius property can be used to choose how rounded the corners of those steps are.
    """
    unique_id = reflex.vars.get_unique_variable_name()
    variable_name = f"smooth_step_path_{unique_id}"

    get_smooth_step_path_call = reflex.vars.FunctionStringVar.create(
        "getSmoothStepPath"
    ).call(
        {
            "sourceX": source_x,
            "sourceY": source_y,
            "targetX": target_x,
            "targetY": target_y,
            "sourcePosition": source_position,
            "targetPosition": target_position,
            "borderRadius": border_radius,
            "centerX": center_x,
            "centerY": center_y,
            "offset": offset,
            "stepPosition": step_position,
        }
    )

    array_name = f"_path_array_{unique_id}"

    result_var = _array_return_to_object(reflex.Var(array_name).to(list))

    return reflex.Var(
        variable_name,
        _var_data=reflex.vars.VarData(
            hooks={
                f"const {array_name} = {get_smooth_step_path_call!s};": get_smooth_step_path_call._get_all_var_data(),
                f"const {variable_name} = {result_var!s};": None,
            },
            imports={LIBRARY: ["getSmoothStepPath"]},
        ),
    ).to(Path)  # pyright: ignore[reportReturnType]


def apply_node_changes(
    nodes: Sequence[Node] | reflex.Var[Sequence[Node]],
    changes: Sequence[NodeChange] | reflex.Var[Sequence[NodeChange]],
) -> reflex.vars.ArrayVar[list[Node]]:
    """Applies changes to nodes in the flow."""
    return (
        reflex.vars.FunctionStringVar.create(
            "applyNodeChanges",
            _var_data=reflex.vars.VarData(
                imports={LIBRARY: ["applyNodeChanges"]},
            ),
        )
        .call(changes, nodes)
        .to(list[Node])
    )


def apply_edge_changes(
    edges: Sequence[Edge] | reflex.Var[Sequence[Edge]],
    changes: Sequence[EdgeChange] | reflex.Var[Sequence[EdgeChange]],
) -> reflex.vars.ArrayVar[list[Edge]]:
    """Applies changes to edges in the flow."""
    return (
        reflex.vars.FunctionStringVar.create(
            "applyEdgeChanges",
            _var_data=reflex.vars.VarData(
                imports={LIBRARY: ["applyEdgeChanges"]},
            ),
        )
        .call(changes, edges)
        .to(list[Edge])
    )


def add_edge(
    params: Edge | Connection | reflex.Var[Edge | Connection],
    edges: Sequence[Edge] | reflex.Var[Sequence[Edge]],
) -> reflex.vars.ArrayVar[list[Edge]]:
    """Creates a new edge in the flow."""
    return reflex.vars.FunctionStringVar.create(
        "addEdge",
        _var_data=reflex.vars.VarData(
            imports={LIBRARY: ["addEdge"]},
        ),
    ).call(params, edges)  # pyright: ignore[reportReturnType]


def get_incomers(
    *, node_id: str, nodes: Sequence[Node], edges: Sequence[Edge]
) -> list[Node]:
    """Returns all incoming nodes connected to the given node."""
    if not node_id:
        return []
    incomers_ids = {edge["source"] for edge in edges if edge["target"] == node_id}
    return [n for n in nodes if n["id"] in incomers_ids]


def get_incomers_var(
    *,
    node_id: str | reflex.Var[str],
    nodes: Sequence[Node] | reflex.Var[Sequence[Node]],
    edges: Sequence[Edge] | reflex.Var[Sequence[Edge]],
) -> reflex.vars.ArrayVar[list[Node]]:
    """Returns all incoming nodes connected to the given node."""
    return (
        reflex.vars.FunctionStringVar.create(
            "getIncomers",
            _var_data=reflex.vars.VarData(
                imports={LIBRARY: ["getIncomers"]},
            ),
        )
        .call({"id": node_id}, nodes, edges)
        .to(list[Node])
    )


def get_outgoers(
    *, node_id: str, nodes: Sequence[Node], edges: Sequence[Edge]
) -> list[Node]:
    """Returns all outgoing nodes connected to the given node."""
    if not node_id:
        return []
    outgoers_ids = {edge["target"] for edge in edges if edge["source"] == node_id}
    return [n for n in nodes if n["id"] in outgoers_ids]


def get_outgoers_var(
    *,
    node_id: str | reflex.Var[str],
    nodes: Sequence[Node] | reflex.Var[Sequence[Node]],
    edges: Sequence[Edge] | reflex.Var[Sequence[Edge]],
) -> reflex.vars.ArrayVar[list[Node]]:
    """Returns all outgoing nodes connected to the given node."""
    return (
        reflex.vars.FunctionStringVar.create(
            "getOutgoers",
            _var_data=reflex.vars.VarData(
                imports={LIBRARY: ["getOutgoers"]},
            ),
        )
        .call({"id": node_id}, nodes, edges)
        .to(list[Node])
    )


def get_connected_edges(*, nodes: Sequence[Node], edges: Sequence[Edge]) -> list[Edge]:
    """Returns all edges connected to the given nodes."""
    node_ids = {node["id"] for node in nodes}
    return [
        edge
        for edge in edges
        if edge["source"] in node_ids or edge["target"] in node_ids
    ]


def get_connected_edges_var(
    *,
    nodes: Sequence[Node] | reflex.Var[Sequence[Node]],
    edges: Sequence[Edge] | reflex.Var[Sequence[Edge]],
) -> reflex.vars.ArrayVar[list[Edge]]:
    """Returns all edges connected to the given nodes."""
    return (
        reflex.vars.FunctionStringVar.create(
            "getConnectedEdges",
            _var_data=reflex.vars.VarData(
                imports={LIBRARY: ["getConnectedEdges"]},
            ),
        )
        .call(nodes, edges)
        .to(list[Edge])
    )



"""Base class for map (leaflet) components."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Callable, Mapping

import reflex as rx
from reflex import ImportVar, constants
from reflex.event import EventSpec
from reflex.vars.base import Var, VarData
from reflex.vars.object import ObjectVar

from reflex_enterprise.components.component import NoSSRComponentEnterprise
from reflex_enterprise.vars import JSAPIVar

PACKAGE_NAME = "react-leaflet"
PACKAGE_VERSION = "5.0.0"

path = rx.asset("MapLazyComponents.jsx", shared=True)
public_path = "$/public" + path

refs_var_data = VarData(
    imports={f"$/{constants.Dirs.STATE_PATH}": [ImportVar(tag="refs")]}
)
refs = Var(
    _js_expr="refs",
    _var_data=refs_var_data,
).to(
    ObjectVar,
    Mapping[str, str],
)


class MapAPIVar(JSAPIVar):
    """Wrapper for the Leaflet Map API object as represented in JS."""


@dataclass
class MapAPI:
    """Map API for leaflet components."""

    ref: str

    @classmethod
    def create(cls, ref: str) -> MapAPI:
        """Create a new MapAPI instance."""
        return cls(ref=ref)

    @property
    def _api(self) -> MapAPIVar:
        return MapAPIVar(
            f"refs['{self.ref}']",
            _var_data=refs_var_data,
        )

    def __getattr__(self, name: str) -> Callable[..., EventSpec]:
        """Get the attribute of the map API."""

        def _call_api(*args, **kwargs):
            """Call the API function with the given arguments."""
            return rx.event.run_script(
                getattr(self._api, name)(*args),
                **kwargs,
            )

        return _call_api


class BaseLeafletComponent(NoSSRComponentEnterprise):
    """Base class for leaflet."""

    library = f"{PACKAGE_NAME}@{PACKAGE_VERSION}"

    lib_dependencies: list[str] = [f"{PACKAGE_NAME}@{PACKAGE_VERSION}", "leaflet@1.9.4"]

    def add_imports(self):
        """Add imports for leaflet components."""
        return {
            "": "leaflet/dist/leaflet.css",
        }


class LazyBaseLeafletComponent(BaseLeafletComponent):
    """Lazy load leaflet components."""

    library = public_path



"""Mantine Dates Demo"""

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


class DateState(rx.State):
    """State for the Dates demo."""

    # Define any state variables you need here
    date: str = ""


# @rx.memo
def picker_grid():
    return rx.grid(
        rx.card(
            rx.text("Calendar"),
            rxe.mantine.dates.calendar(),
        ),
        rx.card(
            rx.text("DateTimePicker"),
            rxe.mantine.dates.date_time_picker(
                label="Pick a Date",
                placeholder="Pick date",
                on_change=lambda value: rx.toast(f"Date selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("DatePicker"),
            rxe.mantine.dates.date_picker(
                on_change=lambda value: rx.toast(f"Date selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("DatePickerInput"),
            rxe.mantine.dates.date_picker_input(
                label="Pick a Date",
                placeholder="Pick date",
                # value=DateState.date,
                on_change=lambda value: rx.toast(f"Date selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("DateInput"),
            rxe.mantine.dates.date_input(),
        ),
        rx.card(
            rx.text("MonthPicker"),
            rxe.mantine.dates.month_picker(
                on_change=lambda value: rx.toast(f"Month selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("MonthPickerInput"),
            rxe.mantine.dates.month_picker_input(
                on_change=lambda value: rx.toast(f"Month selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("YearPicker"),
            rxe.mantine.dates.year_picker(
                on_change=lambda value: rx.toast(f"Year selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("YearPickerInput"),
            rxe.mantine.dates.year_picker_input(
                on_change=lambda value: rx.toast(f"Year selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("TimeInput"),
            rxe.mantine.dates.time_input(
                on_change=lambda value: rx.toast(f"Time selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("TimePicker"),
            rxe.mantine.dates.time_picker(
                on_change=lambda value: rx.toast(f"Time selected: {value}"),
            ),
        ),
        rx.card(
            rx.text("DatePicker w/ presets"),
            rxe.mantine.dates.date_picker(
                on_change=lambda value: rx.toast(f"Date selected: {value}"),
                presets=[
                    {
                        "value": rx.Var(
                            "dayjs().subtract(1, 'day').format('YYYY-MM-DD')"
                        ),
                        "label": "Yesterday",
                    },
                    {"value": rx.Var("dayjs().format('YYYY-MM-DD')"), "label": "Today"},
                    {
                        "value": rx.Var("dayjs().add(1, 'day').format('YYYY-MM-DD')"),
                        "label": "Tomorrow",
                    },
                    {
                        "value": rx.Var("dayjs().add(1, 'month').format('YYYY-MM-DD')"),
                        "label": "Next month",
                    },
                    {
                        "value": rx.Var("dayjs().add(1, 'year').format('YYYY-MM-DD')"),
                        "label": "Next year",
                    },
                    {
                        "value": rx.Var(
                            "dayjs().subtract(1, 'month').format('YYYY-MM-DD')"
                        ),
                        "label": "Last month",
                    },
                    {
                        "value": rx.Var(
                            "dayjs().subtract(1, 'year').format('YYYY-MM-DD')"
                        ),
                        "label": "Last year",
                    },
                ],
            ),
        ),
        # rx.text("TimeGrid"),
        # rxe.mantine.dates.time_grid(),
        # rx.text("TimeValue"),
        # rxe.mantine.dates.time_value(),
        columns="3",
    )


@demo(
    route="/dates",
    title="Dates",
    description="A collection of examples using Mantine Dates components in Reflex.",
)
def dates_page():
    return picker_grid()



"""empty message

Revision ID: 8a1f2192cab7
Revises:
Create Date: 2025-03-04 17:07:14.030943

"""

from typing import Sequence, Union

import sqlalchemy as sa
import sqlmodel
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "8a1f2192cab7"
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "friend",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("age", sa.Integer(), nullable=False),
        sa.Column("years_known", sa.Integer(), nullable=False),
        sa.Column("owes_me", sa.Boolean(), nullable=False),
        sa.Column("has_a_dog", sa.Boolean(), nullable=False),
        sa.Column("spouse_is_annoying", sa.Boolean(), nullable=False),
        sa.Column(
            "met",
            sa.DateTime(timezone=True),
            server_default=sa.text("(CURRENT_TIMESTAMP)"),
            nullable=True,
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("friend")
    # ### end Alembic commands ###



"""PillsInput component for Mantine."""

from typing import Any, Literal

import reflex as rx
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import MantineCoreBase

LiteralPillsInputSize = Literal["xs", "sm", "md", "lg", "xl"]
LiteralPillsInputFieldType = Literal["input", "pills", "cursor"]


class PillsInput(MantineCoreBase):
    """Displays pills in an input field."""

    tag = "PillsInput"

    # Controls input size.
    size: Var[LiteralPillsInputSize]

    # If set, the input is disabled.
    disabled: Var[bool]

    # Displays error message after input.
    error: Var[str]

    # Controls input appearance, defaults to default.
    variant: Var[str]

    # Reference to the input field.
    field_ref: Var[Any]


class PillsInputField(MantineCoreBase):
    """Renders input section."""

    tag = "PillsInput.Field"

    # Determines the type of the field.
    type: Var[LiteralPillsInputFieldType]

    # Show placeholder when field is empty.
    placeholder: Var[str]

    # If set, the input is disabled.
    disabled: Var[bool]


class PillsInputNamespace(rx.ComponentNamespace):
    """Namespace for PillsInput components."""

    __call__ = staticmethod(PillsInput.create)
    field = staticmethod(PillsInputField.create)


pills_input = PillsInputNamespace()



"""SemiCircleProgress component for Reflex."""

from typing import Literal

from reflex.components.component import Component  # noqa: F401
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import MantineLabelProps


class SemiCircleProgress(MantineLabelProps):
    """Display semi circle progress bar."""

    tag = "SemiCircleProgress"

    # Progress value from 0 to 100
    value: Var[int]

    # Key of theme.colors or any valid CSS color value, by default the value is determined based on the color scheme value
    empty_segment_color: Var[str]

    # Key of theme.colors or any valid CSS color value, theme.primaryColor by default
    filled_segment_color: Var[str]

    # Direction from which the circle is filled, 'left-to-right' by default
    fill_direction: Var[Literal["right-to-left", "left-to-right"]]

    # Label position
    label_position: Var[Literal["center", "bottom"]]

    # Orientation of the circle, 'up' by default
    orientation: Var[Literal["up", "down"]]

    # Diameter of the svg in px, 200 by default
    size: Var[str | float]

    # Circle thickness in px, 12 by default
    thickness: Var[float]

    # Transition duration of filled section styles changes in ms, 0 by default
    transition_duration: Var[int]


semi_circle_progress = SemiCircleProgress.create



"""Mantine sub-package."""

from reflex.utils import lazy_loader

from reflex_enterprise import _MANTINE_MAPPING

_SUBMODULES = set()
_SUBMOD_ATTRS = {
    "".join(k.split("components.mantine.")[-1]): v for k, v in _MANTINE_MAPPING.items()
}

_SUBMOD_ATTRS.update(
    {
        "base": ["base"],
    }
)

__getattr__, __dir__, __all__ = lazy_loader.attach(
    __name__,
    submodules=_SUBMODULES,
    submod_attrs=_SUBMOD_ATTRS,
)



"""Action icon demo page."""

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


class ActionIconState(rx.State):
    """State for the action icon demo."""

    count: int = 0

    @rx.event
    def increment(self):
        self.count += 1

    @rx.event
    def decrement(self):
        self.count -= 1


@demo(
    route="/action-icon",
    title="Action Icon",
    description="An action icon is a small, button-like component that has an icon inside.",
)
def action_icon_page():
    """Action icon demo page."""
    return rx.hstack(
        rxe.action_icon(
            rx.icon("minus"),
            on_click=ActionIconState.decrement,
            size="xl",
            variant="gradient",
            disabled=ActionIconState.count <= 0,
            gradient={"from": "red", "to": "orange", "deg": 90},
        ),
        rx.text(f"Count: {ActionIconState.count}"),
        rxe.action_icon(
            rx.icon("plus"),
            on_click=ActionIconState.increment,
            size="xl",
            variant="gradient",
            disabled=ActionIconState.count >= 10,
            gradient={"from": "blue", "to": "cyan", "deg": 90},
        ),
        align="center",
        spacing="4",
    )



"""Type definitions for OIDC authentication in Reflex Enterprise."""

from types import TracebackType
from typing import Any, Callable, Mapping, Protocol, TypedDict


class OIDCUserInfo(TypedDict, total=False):
    """Type describing user info obtained from an OIDC provider.

    Attributes:
        sub: The unique identifier for the user.
    """

    sub: str


class AsyncHTTPResponse(Protocol):
    """Protocol for HTTP response used in OIDCAuthState."""

    status: int

    @property
    def ok(self) -> bool:
        """Indicates if the HTTP response was successful (status code 2xx)."""
        ...

    def raise_for_status(self) -> Any:
        """Raise an exception if the HTTP response indicates an error."""
        ...

    async def json(self) -> Any:
        """Parse and return the JSON content of the HTTP response."""
        ...


class AsyncHTTPResponseContext(Protocol):
    """Protocol for HTTP response used in OIDCAuthState."""

    async def __aenter__(self) -> AsyncHTTPResponse:
        """Enter the asynchronous context manager."""
        ...

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc: BaseException | None,
        tb: TracebackType | None,
    ) -> None:
        """Exit the asynchronous context manager."""
        ...


class AsyncHTTPClientProtocol(Protocol):
    """Protocol for HTTP client used in OIDCAuthState."""

    def get(
        self,
        url: str,
        *,
        headers: Mapping[str, str] | None = None,
    ) -> AsyncHTTPResponseContext:
        """Perform an asynchronous HTTP GET request.

        Args:
            url: The URL to send the GET request to.
            headers: Optional HTTP headers to include in the request.
            data: Optional data to include in the request body (for POST requests).

        Returns:
            An AsyncHTTPResponse object representing the HTTP response.
        """
        ...

    def post(
        self,
        url: str,
        *,
        headers: Mapping[str, str] | None = None,
        data: Any = None,
    ) -> AsyncHTTPResponseContext:
        """Perform an asynchronous HTTP POST request.

        Args:
            url: The URL to send the POST request to.
            headers: Optional HTTP headers to include in the request.
            data: Optional data to include in the request body.

        Returns:
            An AsyncHTTPResponse object representing the HTTP response.
        """
        ...


AsyncHTTPClientGetter = Callable[[], AsyncHTTPClientProtocol]



import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.flow.types import Edge, Node

initial_nodes: list[Node] = [
    {
        "id": "1",
        "data": {"label": "Node 1"},
        "position": {"x": 0, "y": 0},
        "style": {
            "width": 200,
            "height": 100,
        },
    },
    {
        "id": "2",
        "data": {"label": "Node 2"},
        "position": {"x": 0, "y": 150},
    },
    {
        "id": "3",
        "data": {"label": "Node 3"},
        "position": {"x": 250, "y": 0},
    },
    {
        "id": "4",
        "data": {"label": "Node"},
        "position": {"x": 350, "y": 150},
        "style": {
            "width": 50,
            "height": 50,
        },
    },
]

initial_edges: list[Edge] = []


class IntersectionsState(rx.State):
    nodes: rx.Field[list[Node]] = rx.field(default_factory=lambda: initial_nodes)
    edges: rx.Field[list[Edge]] = rx.field(default_factory=lambda: initial_edges)

    @rx.event
    def set_nodes(self, nodes: list[Node]):
        self.nodes = nodes

    @rx.event
    def set_edges(self, edges: list[Edge]):
        self.edges = edges


@rx.page(route="/nodes/intersections", title="Intersections")
def intersections():
    return rx.box(
        rxe.flow.provider(
            rxe.flow(
                rxe.flow.background(),
                rxe.flow.controls(),
                on_nodes_change=lambda node_changes: IntersectionsState.set_nodes(
                    rxe.flow.util.apply_node_changes(
                        IntersectionsState.nodes, node_changes
                    )
                ),
                on_edges_change=lambda edge_changes: IntersectionsState.set_edges(
                    rxe.flow.util.apply_edge_changes(
                        IntersectionsState.edges, edge_changes
                    )
                ),
                on_node_drag=lambda node: IntersectionsState.set_nodes(
                    IntersectionsState.nodes.foreach(
                        lambda n: n.to(Node).merge(
                            {
                                "className": rx.cond(
                                    rxe.flow.api.get_intersecting_nodes(node)
                                    .foreach(lambda n: n["id"])
                                    .contains(n["id"]),
                                    "highlight",
                                    "",
                                )
                            }
                        )
                    ),
                ).throttle(100),
                nodes=IntersectionsState.nodes,
                edges=IntersectionsState.edges,
                color_mode="light",
                class_name="intersection-flow",
                select_nodes_on_drag=False,
                fit_view=True,
            ),
        ),
        height="100vh",
        width="100vw",
    )



"""Example of handling grid selection events and displaying selected items in a separate panel."""

import pandas as pd
import reflex as rx

import reflex_enterprise as rxe

from .common import demo

df = pd.read_csv(
    "https://raw.githubusercontent.com/plotly/datasets/master/wind_dataset.csv"
)

column_defs = [
    rxe.ag_grid.column_def(field="direction"),  # pyright: ignore [reportCallIssue]
    rxe.ag_grid.column_def(field="strength"),  # pyright: ignore [reportCallIssue]
    rxe.ag_grid.column_def(field="frequency"),  # pyright: ignore [reportCallIssue]
]


class BasicGridState(rx.State):
    """State for the basic grid example."""

    selection: list[dict[str, str]] = []

    keys: list[str] = []

    @rx.event
    def set_keys(self, keys: list[str]):
        self.keys = [k.upper() for k in keys]
        return type(self).select_by_direction

    @rx.event
    def select_by_direction(self):
        return grid_api.select_rows_by_key(self.keys, node_path_key="data.direction")


grid = rxe.ag_grid(
    id="ag_grid_basic_1",
    row_data=df.to_dict("records"),  # pyright: ignore [reportArgumentType]
    column_defs=column_defs,  # pyright: ignore [reportArgumentType]
    row_selection={"mode": "multiRow"},
    on_selection_changed=lambda rows, _0, _1: BasicGridState.setvar(  # pyright: ignore [reportArgumentType]
        "selection", rows
    ),
    width="50%",
    height="71vh",
)
grid_api = rxe.ag_grid.api("ag_grid_basic_1")


def selected_item(item: dict[str, str]) -> rx.Component:
    """Create a selected item component."""
    return rx.card(
        rx.data_list.root(
            rx.foreach(
                item,
                lambda kv: rx.data_list.item(
                    rx.data_list.label(kv[0]),
                    rx.data_list.value(kv[1]),
                ),
            ),
        ),
    )


@demo(
    route="/selected-items",
    title="Selected Items",
    description="Handle grid selection events and display selected items in a separate panel.",
)
def selected_items_example():
    """Selected items example."""
    return rx.hstack(
        rx.vstack(
            rx.text("Select by Direction"),
            rxe.mantine.tags_input(
                value=BasicGridState.keys,
                allow_duplicates=False,
                on_change=BasicGridState.set_keys,
                width="10vw",
            ),
            rx.button(
                "Select All",
                on_click=[grid_api.select_all],  # pyright: ignore [reportAttributeAccessIssue]
            ),
            rx.button(
                "Deselect All",
                on_click=[grid_api.deselect_all, BasicGridState.set_keys([])],  # pyright: ignore [reportAttributeAccessIssue]
            ),
            rx.button("Log nodes", on_click=grid_api.log_nodes("data")),
        ),
        grid,
        rx.vstack(
            rx.heading(
                f"Selected Items ({BasicGridState.selection.length()})",  # pyright: ignore [reportAttributeAccessIssue]
                size="4",
            ),
            rx.scroll_area(
                rx.hstack(
                    rx.foreach(
                        BasicGridState.selection,
                        selected_item,
                    ),
                    wrap="wrap",
                ),
            ),
            max_width="48%",
            height="71vh",
        ),
        width="100%",
    )



"""Demo of a customized ModelWrapper with auth state."""

from typing import Any

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.ag_grid.datasource import DatasourceParams

from .common import demo
from .model_wrapper_simple import Friend


# This bogus auth state demonstrates how an extended ModelWrapper can check
# against values in another state before returning/modifying the data.
class AuthState(rx.State):
    """State for the auth demo."""

    _logged_in: bool = False

    @rx.var
    def logged_in(self) -> bool:
        """Whether the user is logged in."""
        return self._logged_in

    @rx.event
    def toggle_login(self):
        """Toggle the login state."""
        self._logged_in = not self._logged_in
        return self._refresh_grid()

    def _refresh_grid(self):
        from .model_wrapper_ssrm import FriendModelWrapperSSRM

        return [
            cls.on_mount
            for cls in FriendModelWrapper.__subclasses__()
            + FriendModelWrapperSSRM.__subclasses__()
        ]

    @rx.event
    async def generate_friends(self, n: int):
        """If only it were that easy..."""
        from .model_wrapper_ssrm import FriendModelWrapperSSRM

        if not self._logged_in:
            yield rx.toast.error("You must be logged in to generate friends.")
            return
        with rx.session() as session:
            for f in Friend.generate_fakes(n):
                session.add(f)
            session.commit()
        yield rx.toast.info(f"Created {n} friends.")
        for cls in (
            FriendModelWrapper.__subclasses__()
            + FriendModelWrapperSSRM.__subclasses__()
        ):
            inst = await self.get_state(cls)
            yield cls._grid_component.api.set_row_count(await inst._row_count())


# When extending a ModelWrapper, you can override methods to customize the behavior.
class FriendModelWrapper(rxe.ModelWrapper[Friend]):
    """Customized ModelWrapper for the Friend model."""

    def _get_column_defs(self):
        """In this example, we remove the ability to filter and sort a particular field."""
        cols = super()._get_column_defs()
        for col in cols:
            if col.field == "spouse_is_annoying":
                col.filter = None
                col.sortable = False
        return cols

    @rx.event
    async def on_value_setter(
        self, row_data: dict[str, Any], field_name: str, value: Any
    ):
        """In this example, we prevent modifications to the data if the user is not logged in."""
        auth_state = await self.get_state(AuthState)
        if not auth_state.logged_in:
            return  # no modification for logged out users
        return await super().on_value_setter(row_data, field_name, value)

    async def _get_data(
        self,
        params: DatasourceParams,
    ) -> list[Friend]:
        auth_state = await self.get_state(AuthState)
        if not auth_state.logged_in:
            return []  # no records for logged out users
        return await super()._get_data(params)

    @rx.var
    def selected_items(self) -> list[Friend]:
        """Get the selected items from the grid."""
        # Normally selected items are backend-only, but we can provide
        # a computed var to render them in the UI.
        return self._selected_items


# Advanced example of an extended ModelWrapper with custom behavior
@demo(
    route="/model-auth",
    title="Customized ModelWrapper",
    description="Extended infinite-row ModelWrapper with custom behavior and auth.",
)
def model_page_auth():
    """Page for the customized ModelWrapper demo."""
    grid = FriendModelWrapper.create(
        model_class=Friend,
        row_selection={"mode": "multiRow"},
    )
    return rx.vstack(
        rx.hstack(
            rx.cond(
                AuthState.logged_in,
                rx.button("Logout", on_click=AuthState.toggle_login),
                rx.button("Login", on_click=AuthState.toggle_login),
            ),
            rx.cond(
                AuthState.logged_in,
                rx.button("Generate Friends", on_click=AuthState.generate_friends(50)),
            ),
            rx.foreach(
                grid.State.selected_items,  # pyright: ignore [reportAttributeAccessIssue]
                lambda friend: rx.badge(friend.name),
            )
            if grid.State
            else (),
            rx.spacer(),
        ),
        rx.box(
            grid,
            width="100%",
            height="65vh",
            padding_bottom="60px",  # for scroll bar and controls
        ),
    )



"""Editable AG Grid Example."""

from typing import TypedDict

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


class Person(TypedDict):
    """Person type for AG Grid."""

    name: str
    age: int
    country: str


class EditableState(rx.State):
    """State for the editable demo."""

    # This state is used to store the data for the AG Grid.
    row_data: list[Person] = [
        {"name": "John", "age": 30, "country": "USA"},
        {"name": "Anna", "age": 25, "country": "Sweden"},
        {"name": "Mike", "age": 35, "country": "Canada"},
    ]

    @rx.event
    def on_cell_value_changed(self, params: dict[str, str]):
        """Handle cell value changes."""
        # We use node_id which represent the initial position of the changed row in the dataset.
        # rowIndex is the index of the row in the grid, which may change if the grid is sorted or filtered.
        row_index = int(params["node_id"])
        field = params["field"]
        new_value = params["newValue"]

        # Update the row data in the state.
        self.row_data[row_index][field] = new_value

        # Show a toast message with the updated value.
        return rx.toast(f"Cell value changed: {field} = {new_value}")


column_defs = [
    {"header_name": "Name", "field": "name", "editable": True},
    {"header_name": "Age", "field": "age", "editable": True},
    {"header_name": "Country", "field": "country", "editable": True},
]


@demo(
    route="/editable",
    title="Editable AG Grid",
    description="An editable AG Grid example.",
)
def editable_page():
    """Editable AG Grid example page."""
    return rxe.ag_grid(
        id="editable-grid",
        column_defs=column_defs,
        row_data=EditableState.row_data,  # pyright: ignore [reportArgumentType],
        on_cell_value_changed=EditableState.on_cell_value_changed,  # pyright: ignore [reportArgumentType],
    )



"""ag-grid rx.model wrapper."""

from __future__ import annotations

import datetime
import enum
import json
from typing import Any, ClassVar, Generic, Sequence, Type, get_args

import reflex as rx
from reflex.event import EventCallback, EventSpec
from reflex.utils import console
from reflex.utils.serializers import serialize
from reflex.utils.types import is_optional, is_union
from reflex.vars.function import FunctionStringVar
from sqlalchemy.inspection import inspect
from sqlmodel import col, func, select
from starlette.requests import Request
from starlette.responses import JSONResponse

from reflex_enterprise.components.ag_grid import ag_grid
from reflex_enterprise.components.ag_grid.datasource import (
    DATASOURCE_PARAMS,
    Datasource,
    DatasourceParams,
    LoadSuccessParams,
    SSRMDatasource,
    SSRMDatasourceRequestParams,
)
from reflex_enterprise.components.ag_grid.handlers import (
    M,
    apply_filter_model,
    apply_sort_model,
)
from reflex_enterprise.components.ag_grid.resources import ColumnDef


class ReflexJSONResponse(JSONResponse):
    """Custom JSONResponse that uses Reflex's serializer."""

    def render(self, content: Any) -> bytes:
        """Render content using Reflex's serializer."""
        return json.dumps(content, default=serialize).encode("utf-8")


def _value_setter_signature(
    params: rx.Var[dict[str, Any]],
) -> tuple[rx.Var[dict], rx.Var[str], rx.Var[Any]]:
    return (
        params["data"],  # pyright: ignore [reportIndexIssue]
        params["colDef"].to(dict)["field"],  # pyright: ignore [reportIndexIssue]
        params["newValue"],  # pyright: ignore [reportIndexIssue]
    )


def get_default_column_def(
    field: str,
    ftype: Type,
    value_setter: rx.EventHandler | EventCallback | None = None,
    **cdef_kwargs,
) -> ColumnDef:
    """Get a default column definition for a given field type.

    Args:
        field: The field name.
        ftype: The field type.
        value_setter: The value setter event handler.
        **cdef_kwargs: Additional ColumnDef keyword arguments.

    Returns:
        The column definition.
    """
    _cdef_kwargs = {
        "sortable": True,
        "filter": True,
        "editable": value_setter is not None,
        "cell_editor": ag_grid.editors.text,
    }
    _cdef_kwargs.update(cdef_kwargs)
    cdef = ag_grid.column_def(
        field=field,
        **_cdef_kwargs,
    )
    if is_optional(ftype):
        ftype = get_args(ftype)[0]
    if ftype in (int, float, bool):
        cdef.type = "numericColumn"
        cdef.filter = ag_grid.filters.number
        cdef.cell_editor = ag_grid.editors.number
        cdef.cell_data_type = "number"
    if ftype is bool:
        cdef.cell_renderer = ag_grid.renderers.checkbox_cell
        cdef.cell_editor = ag_grid.editors.checkbox
        cdef.cell_data_type = "boolean"
    if ftype is datetime.datetime:
        cdef.filter = ag_grid.filters.date
        cdef.cell_editor = ag_grid.editors.date
    if value_setter is not None:
        cdef.value_setter = rx.EventChain(
            events=[
                rx.event.call_event_handler(
                    value_setter,  # pyright: ignore [reportArgumentType]
                    _value_setter_signature,
                ),
            ],
            args_spec=_value_setter_signature,
            # XXX: hacks to queue events from call_script eval context
            invocation=rx.vars.function.ArgsFunctionOperation.create(
                args_names=["events"],
                return_expr=rx.Var(
                    "queueEvents(events, {current: socket}, false, navigate, params)"
                ),
            ),
        )
    return cdef


class AbstractWrapper(rx.ComponentState, Generic[DATASOURCE_PARAMS]):
    """Abstract class for wrapping ag-grid for infinite data model."""

    _grid_component: ClassVar[rx.Component | None] = None
    _selected_items: list[Any] = []

    __data_route__: ClassVar[str] = "/abstract-wrapper-data"
    __get_data_kwargs__: ClassVar[dict[str, Any]] = {
        "state": lambda self: self.get_full_name()
    }
    __data_source_params_class__: Type[DATASOURCE_PARAMS] = DatasourceParams  # pyright: ignore [reportAssignmentType]

    @classmethod
    def _add_data_route(cls):
        """Add the backend __data_route__ that responds to ag-grid data requests.

        The backend route will call the _get_data method to fetch the data.
        """
        app = rx.utils.prerequisites.get_app().app  # pyright: ignore [reportAttributeAccessIssue]

        # Check if route already exists
        for route in app._api.router.routes:
            if hasattr(route, "path") and route.path == cls.__data_route__:
                return

        async def get_data(request: Request):
            # Extract state from query params
            state = request.query_params.get("state", "")

            try:
                token = request.headers["X-Reflex-Client-Token"]
            except KeyError:
                return ReflexJSONResponse([])
            state_cls = rx.State.get_class_substate(tuple(state.split(".")))
            # Get the correct params class from the actual state class, not the closure
            params = state_cls.__data_source_params_class__.from_request(  # pyright: ignore [reportAttributeAccessIssue, reportArgumentType]
                request.query_params
            )
            async with app.modify_state(
                rx.state._substate_key(token, state_cls)
            ) as _state:
                s_instance = await _state.get_state(state_cls)
                result = s_instance._get_data(params)
                if hasattr(result, "__await__"):
                    result = await result
                return ReflexJSONResponse(result)

        # Add route using Starlette's add_route method
        app._api.add_route(cls.__data_route__, get_data, methods=["GET"])

    def _get_datasource_kwargs(self) -> dict[str, Any]:
        """Get additional kwargs to pass in the datasource URI -- accessed during on_mount."""
        return {
            key: value if not callable(value) else value(self)
            for key, value in self.__get_data_kwargs__.items()
        }

    async def _set_datasource(self) -> EventSpec:
        """Set the datasource for the grid.

        This is called during on_mount.
        """
        ds = Datasource(
            endpoint_uri=self.__data_route__,
            endpoint_kwargs=self._get_datasource_kwargs(),
            rowCount=await self._row_count(),
        )
        return self._grid_component.api.set_grid_option(  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
            "datasource", ds
        )

    @rx.event
    async def on_mount(self):
        """Perform post-hydration grid initialization.

        Set up column defs and data source to fetch infinite row data.
        """
        return [
            self._grid_component.api.set_grid_option(  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
                "columnDefs", self._get_column_defs()
            ),
            await self._set_datasource(),
        ]

    @rx.event
    async def refresh(self):
        """Refresh the grid data."""
        return await self.on_mount()

    @rx.event
    def on_selection_changed(self, rows: list[dict], source: dict, type: str):
        """Handle selection changed event."""
        self._selected_items = rows

    @rx.event
    def on_value_setter(self, row_data: dict[str, Any], field_name: str, value: Any):
        """Handle setting value in the model."""
        raise NotImplementedError("Handle setting value in the model.")

    def _get_column_defs(self) -> list[ColumnDef]:
        """Get the column definitions for the grid, must be overridden."""
        raise NotImplementedError("Handle fetching column definitions.")

    def _get_data(
        self,
        params: DATASOURCE_PARAMS,
    ) -> list[dict[str, Any]]:
        """Get the data for the grid, must be overridden."""
        raise NotImplementedError("Handle fetching data from the model.")

    async def _row_count(self) -> int:
        """Get the total row count for the grid, must be overridden."""
        raise NotImplementedError("Handle fetching row count.")

    @classmethod
    def _default_grid_props(cls) -> dict[str, Any]:
        return {
            "id": f"ag-grid-{cls.get_full_name()}",
            "default_col_def": {"flex": 1},
            "max_blocks_in_cache": 4,
            "cache_block_size": 50,
            "group_default_expanded": None,
            "row_model_type": "infinite",
        }

    @classmethod
    def get_component(cls, *children, **props) -> rx.Component:
        """Return the Ag-Grid component linked to the wrapper state.

        Args:
            children: The children components passed to ag_grid, typically not used.
            **props: Additional props for the ag_grid component.

        Note that "column_defs", "row_model_type", "on_mount", and
        "on_selection_changed" cannot be passed here.

        Returns:
            The Ag-Grid component.
        """
        _props = cls._default_grid_props()
        _props.update(props)
        return ag_grid.root(
            *children,
            on_mount=cls.on_mount,
            on_selection_changed=cls.on_selection_changed,  # pyright: ignore [reportArgumentType]
            **_props,
        )

    @classmethod
    def create(cls, *children, **props) -> rx.Component:
        """Create the ComponentState instance.

        Args:
            children: The children components passed to ag_grid, typically not used.
            **props: Additional props for the ag_grid component.

        Note that "column_defs", "row_model_type", "on_mount", and
        "on_selection_change" cannot be passed here.

        Returns:
            The Ag-Grid component linked to the wrapper state.
        """
        comp = super().create(*children, **props)
        comp.State._grid_component = comp  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
        comp.State._add_data_route()  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
        return comp


class ModelWrapperActionType(enum.Enum):
    """ModelWrapper action types."""

    SELECT = "select"
    INSERT = "insert"
    UPDATE = "update"
    DELETE = "delete"


class ModelWrapperBase(
    AbstractWrapper[DATASOURCE_PARAMS], Generic[DATASOURCE_PARAMS, M]
):
    """Ag-Grid wrapper for arbitrary rx.Model class."""

    _model_class: ClassVar[Type[M] | None] = None  # pyright: ignore [reportGeneralTypeIssues]
    _primary_key: ClassVar[tuple[str, ...]] = ("id",)
    _selected_items: list[M] = []
    add_dialog_is_open: bool = False

    async def _is_authorized(
        self,
        action: ModelWrapperActionType,
        action_data: Sequence[M] | dict[str, Any] | None,
    ) -> bool:
        """Check if the user is authorized to perform the action.

        For SELECT, action_data is None.
        For INSERT, action_data is a dictionary of the new row data.
        For UPDATE, action data is a dictionary of updated row data.
        For DELETE, action data is a list of model objects to delete.

        Args:
            action: The action type.
            action_data: The data associated with the action.

        Returns:
            True if authorized, False otherwise.
        """
        return True

    @rx.event
    def on_selection_changed(self, rows: list[dict], source: Any, type: Any):
        """Handle selection changed event."""
        if self._model_class:
            self._selected_items = [self._model_class(**row) for row in rows]

    @classmethod
    def _annotation_for(cls, fname: str) -> Type:
        if cls._model_class is None:
            return type(None)
        field = cls._model_class.__fields__[fname]
        type_ = field.type_ if hasattr(field, "type_") else field.annotation  # pyright: ignore [reportAttributeAccessIssue]
        if is_optional(type_) or is_union(type_):
            type_ = get_args(type_)[0]
        return type_  # pyright: ignore [reportReturnType]

    @classmethod
    def _default_sort_model(cls) -> list[dict[str, str]]:
        return [{"colId": pkf, "sort": "asc"} for pkf in cls._primary_key]

    @rx.event
    async def on_value_setter(
        self, row_data: dict[str, Any], field_name: str, value: Any
    ):
        """Handles setting a value in the model."""
        if not self._model_class:
            return

        if not await self._is_authorized(
            ModelWrapperActionType.UPDATE, row_data | {field_name: value}
        ):
            return
        try:
            if self._annotation_for(field_name) == datetime.datetime:
                value = datetime.datetime.fromisoformat(value)
        except KeyError:
            pass

        async with rx.asession() as session:
            item_orm = await session.get(
                self._model_class,
                tuple(
                    self._annotation_for(pkf)(row_data[pkf])
                    for pkf in self._primary_key
                ),
            )
            if item_orm is not None:
                setattr(item_orm, field_name, value)
                session.add(item_orm)
                await session.commit()
                return await self.refresh()

    @rx.event
    async def on_add(self, row_data: dict[str, Any]):
        """Handles submitting a new row to the model."""
        if not self._model_class:
            return
        if not await self._is_authorized(ModelWrapperActionType.INSERT, row_data):
            return
        async with rx.asession() as session:
            item = self._model_class(**row_data)
            session.add(item)
            await session.commit()
            self.add_dialog_is_open = False
            return await self.refresh()

    @rx.event
    async def delete_selected(self):
        """Handles deleting selected rows from the model."""
        if not self._model_class:
            return
        if not await self._is_authorized(
            ModelWrapperActionType.DELETE, self._selected_items
        ):
            return
        async with rx.asession() as session:
            for item in self._selected_items:
                item = await session.get(
                    self._model_class,
                    tuple(getattr(item, pkf) for pkf in self._primary_key),
                )  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
                await session.delete(item)
            await session.commit()
            self._selected_items = []  # they're deleted now.
            return [
                self._grid_component.api.deselect_all(),  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
                await self.refresh(),
            ]

    def _get_column_defs(self) -> list[ColumnDef]:
        if not self._model_class:
            return []
        return [
            get_default_column_def(
                field=field.name if hasattr(field, "name") else field_name,  # pyright: ignore [reportAttributeAccessIssue]
                ftype=field.type_ if hasattr(field, "type_") else field.annotation,  # pyright: ignore [reportAttributeAccessIssue,reportArgumentType]
                value_setter=type(self).on_value_setter,
                editable=field_name not in self._primary_key,
            )
            for field_name, field in self._model_class.__fields__.items()
        ]

    async def _row_count(self) -> int:
        if not self._model_class:
            return 0
        async with rx.asession() as session:
            return (
                await session.exec(
                    select(
                        func.count(
                            col(getattr(self._model_class, self._primary_key[0]))
                        )
                    )
                )
            ).one()

    @rx.var(interval=10, auto_deps=False, initial_value=0)
    async def row_count_cached(self) -> int:
        """Get the cached row count for the grid."""
        return await self._row_count()

    @classmethod
    def _add_dialog_field(cls, field: str, ftype: Type) -> rx.Component:
        comp = rx.input(name=field)
        orig_type_str = str(ftype)
        if is_optional(ftype):
            ftype = get_args(ftype)[0]
        if ftype in (int, float):
            comp = rx.input(
                name=field,
                type="number",
            )
        if ftype is bool:
            comp = rx.checkbox(id=field)
        return rx.table.row(
            rx.table.cell(rx.text(field)),
            rx.table.cell(comp),
            rx.table.cell(rx.text(f"({orig_type_str})")),
        )

    @classmethod
    def _add_dialog(cls) -> rx.Component:
        """Create the dialog for adding a new row."""
        if not cls._model_class:
            return rx.text("No model class specified.")
        return rx.dialog.root(
            rx.dialog.trigger(
                rx.icon_button("plus"),
            ),
            rx.dialog.content(
                rx.dialog.title(f"Add new {cls._model_class.__name__}"),
                rx.form(
                    rx.table.root(
                        *[
                            cls._add_dialog_field(
                                field.name if hasattr(field, "name") else field_name,  # pyright: ignore [reportAttributeAccessIssue]
                                field.type_  # pyright: ignore [reportAttributeAccessIssue,reportArgumentType]
                                if hasattr(field, "type_")
                                else field.annotation,  # pyright: ignore [reportAttributeAccessIssue]
                            )
                            for field_name, field in cls._model_class.__fields__.items()
                            if field_name not in cls._primary_key
                        ]
                    ),
                    rx.button("Add", margin="5px"),
                    on_submit=cls.on_add,
                ),
            ),
            open=cls.add_dialog_is_open,
            on_open_change=cls.setvar("add_dialog_is_open"),
        )

    @classmethod
    def _delete_button(cls) -> rx.Component:
        """Create the delete button."""
        return rx.icon_button(
            "trash-2",
            on_click=cls.delete_selected,
        )

    @classmethod
    def _top_toolbar(cls) -> rx.Component:
        """Create the top toolbar."""
        return rx.hstack(
            cls._delete_button(),
            cls._add_dialog(),
            justify="end",
            margin="5px",
        )

    @classmethod
    def get_component(cls, *children, model_class: Type[M], **props) -> rx.Component:
        """Get the component for the specified model class."""
        cls._model_class = model_class  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
        cls._primary_key = tuple(  # pyright: ignore [reportAttributeAccessIssue]
            key.name
            for key in inspect(model_class).primary_key  # pyright: ignore [reportOptionalMemberAccess]
        )
        return super().get_component(*children, **props)

    @classmethod
    def create(cls, *children, model_class: Type[M], **props) -> rx.Component:
        """Create the Wrapper instance."""
        comp = super().create(*children, model_class=model_class, **props)
        if not comp.State:
            console.debug(
                f"No State class found for ModelWrapper of {model_class.__class__.__name__}"
            )
            return comp
        frag = rx.fragment(
            comp.State._top_toolbar(),  # pyright: ignore [reportAttributeAccessIssue]
            comp,
            State=comp.State,
        )
        frag.api = comp.api  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
        return frag


class ModelWrapper(ModelWrapperBase[DatasourceParams, M], Generic[M]):
    """Wrapper for arbitrary rx.Model class with infinite row model."""

    @rx.event
    async def refresh(self):
        """Refresh the grid data."""
        if not self._model_class:
            return
        return self._grid_component.api.refreshInfiniteCache()  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]

    async def _get_data(
        self,
        params: DatasourceParams,
    ) -> Sequence[M] | tuple[Sequence[M], int]:
        if not self._model_class:
            return []
        if not await self._is_authorized(ModelWrapperActionType.SELECT, None):
            return []
        async with rx.asession() as session:
            result = await session.exec(  # pyright: ignore [reportReturnType]
                apply_sort_model(
                    model=self._model_class,
                    query=apply_filter_model(
                        model=self._model_class,
                        filter_model=params.filterModel or {},
                    ),
                    sort_model=params.sortModel or self._default_sort_model(),
                )
                .offset(params.startRow)
                .limit(params.endRow - params.startRow)
            )
            return result.all(), await self.row_count_cached


class ModelWrapperSSRM(ModelWrapperBase[SSRMDatasourceRequestParams, M], Generic[M]):
    """Wrapper for arbitrary rx.Model class with SSRM."""

    __data_source_params_class__: DATASOURCE_PARAMS = SSRMDatasourceRequestParams  # pyright: ignore [reportIncompatibleVariableOverride, reportGeneralTypeIssues]

    @classmethod
    def _default_grid_props(cls) -> dict[str, Any]:
        row_id_parts = [f"${{params.data['{key}']}}" for key in cls._primary_key]
        return {
            **super()._default_grid_props(),
            "row_model_type": "serverSide",
            "get_row_id": FunctionStringVar(f"(params) => `{''.join(row_id_parts)}`"),
            "enterprise_modules": {
                "ServerSideRowModelModule",
                "ServerSideRowModelApiModule",
            },
        }

    async def _set_datasource(self) -> EventSpec:
        return self._grid_component.api.set_grid_option(  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]
            "serverSideDatasource",
            SSRMDatasource(
                endpoint_uri=self.__data_route__,
                endpoint_kwargs=self._get_datasource_kwargs(),
                rowCount=0,
            ),
        )

    @rx.event
    async def refresh(self):
        """Refresh the grid data."""
        if not self._model_class:
            return
        return self._grid_component.api.refreshServerSide()  # pyright: ignore [reportAttributeAccessIssue, reportOptionalMemberAccess]

    async def _get_data(
        self,
        params: SSRMDatasourceRequestParams,
    ) -> Sequence[M] | LoadSuccessParams[M]:
        if not self._model_class:
            return []
        if not await self._is_authorized(ModelWrapperActionType.SELECT, None):
            return []
        async with rx.asession() as session:
            result = await session.exec(  # pyright: ignore [reportReturnType]
                apply_sort_model(
                    model=self._model_class,
                    query=apply_filter_model(
                        model=self._model_class,
                        filter_model=params.filterModel or {},
                    ),
                    sort_model=params.sortModel or self._default_sort_model(),
                )
                .offset(params.start_row)
                .limit(params.end_row - params.start_row)
            )
            return {
                "rowData": result.all(),
                "rowCount": await self.row_count_cached,
            }


model_wrapper = ModelWrapper.create
model_wrapper_ssrm = ModelWrapperSSRM.create



"""AG Grid Demo."""

import reflex as rx

import reflex_enterprise as rxe

# from .accordion_demo import accordion_page
# from .action_icon_demo import action_icon_page
# from .alert_demo import alert_page
# from .anchor_demo import anchor_page
# from .angle_slider_demo import angle_slider_page
# from .aspect_ratio_demo import aspect_ratio_page
# from .combobox import combobox_page
from .common import DemoState, demo
from .dates import dates_page
from .pill_demo import pill_page
from .tags_input import tags_input_page

__all__ = [
    # "accordion_page",
    # "action_icon_page",
    # "alert_page",
    # "anchor_page",
    # "angle_slider_page",
    "app",
    # "combobox_page",
    # "aspect_ratio_page",
    "dates_page",
    "pill_page",
    "tags_input_page",
]


@demo(
    route="/",
    title="Mantine Demo",
    description="A collection of examples using Mantine in Reflex.",
)
def index():
    """Index page for the AG Grid demos."""
    return rx.flex(
        rx.foreach(
            DemoState.pages,
            lambda page: rx.card(
                rx.vstack(
                    rx.link(page.title, href=page.route),
                    rx.text(page.description),
                ),
                width="300px",
            ),
        ),
        wrap="wrap",
        spacing="3",
    )


# Add state and page to the app.
app = rxe.App()

# Uncomment this when deploying demos, to initialize the database.
# rx.utils.prerequisites.check_db_initialized() or rx.Model.alembic_init()
# rx.Model.migrate(autogenerate=True) #



"""Vector layer components for Leaflet maps."""

from __future__ import annotations

from typing import Dict, List, TypedDict

from reflex.vars.base import Var

from .base import BaseLeafletComponent
from .types import LatLng, LatLngBounds


class PathOptions(TypedDict, total=False):
    """Options for styling vector paths."""

    stroke: bool  # Whether to draw stroke along the path
    color: str  # Stroke color
    weight: int  # Stroke width in pixels
    opacity: float  # Stroke opacity
    lineCap: str  # Shape of stroke ends ('butt', 'round' or 'square')
    lineJoin: str  # Shape of stroke corners ('miter', 'round' or 'bevel')
    dashArray: str  # String defining dash pattern (e.g. '5, 10')
    dashOffset: str  # Distance into the dash pattern to start
    fill: bool  # Whether to fill the path with color
    fillColor: str  # Fill color
    fillOpacity: float  # Fill opacity
    fillRule: str  # Fill rule ('nonzero' or 'evenodd')
    className: str  # Custom CSS class name


def path_options(
    *,
    stroke: bool | None = None,
    color: str | None = None,
    weight: int | None = None,
    opacity: float | None = None,
    line_cap: str | None = None,  # 'butt', 'round' or 'square'
    line_join: str | None = None,  # 'miter', 'round' or 'bevel'
    dash_array: str | None = None,
    dash_offset: str | None = None,
    fill: bool | None = None,
    fill_color: str | None = None,
    fill_opacity: float | None = None,
    fill_rule: str | None = None,  # 'nonzero' or 'evenodd'
    class_name: str | None = None,
) -> PathOptions:
    """Create a PathOptions dictionary with styling options for vector paths.

    Args:
        stroke: Whether to draw stroke along the path
        color: Stroke color
        weight: Stroke width in pixels
        opacity: Stroke opacity
        line_cap: Shape of stroke ends ('butt', 'round' or 'square')
        line_join: Shape of stroke corners ('miter', 'round' or 'bevel')
        dash_array: String defining dash pattern (e.g. '5, 10')
        dash_offset: Distance into the dash pattern to start
        fill: Whether to fill the path with color
        fill_color: Fill color
        fill_opacity: Fill opacity
        fill_rule: Fill rule ('nonzero' or 'evenodd')
        class_name: Custom CSS class name

    Returns:
        A dictionary with path styling options
    """
    options = {}

    if stroke is not None:
        options["stroke"] = stroke
    if color is not None:
        options["color"] = color
    if weight is not None:
        options["weight"] = weight
    if opacity is not None:
        options["opacity"] = opacity
    if line_cap is not None:
        options["lineCap"] = line_cap
    if line_join is not None:
        options["lineJoin"] = line_join
    if dash_array is not None:
        options["dashArray"] = dash_array
    if dash_offset is not None:
        options["dashOffset"] = dash_offset
    if fill is not None:
        options["fill"] = fill
    if fill_color is not None:
        options["fillColor"] = fill_color
    if fill_opacity is not None:
        options["fillOpacity"] = fill_opacity
    if fill_rule is not None:
        options["fillRule"] = fill_rule
    if class_name is not None:
        options["className"] = class_name

    return options  # type: ignore[return-value]


class Circle(BaseLeafletComponent):
    """Circle component for displaying a circle on a map."""

    tag = "Circle"

    # The center of the circle
    center: Var[LatLng]

    # The radius of the circle in meters
    radius: Var[float]

    # Styling options for the circle
    path_options: Var[PathOptions]

    # The attribution text
    attribution: Var[str]

    # The map pane where the circle will be added
    pane: Var[str]


class CircleMarker(BaseLeafletComponent):
    """CircleMarker component for displaying a small circle marker on a map.

    Unlike Circle, CircleMarker uses screen pixels for its radius, not meters.
    """

    tag = "CircleMarker"

    # The center of the circle marker
    center: Var[LatLng]

    # The radius of the circle marker in pixels
    radius: Var[float]

    # Styling options for the circle marker
    path_options: Var[PathOptions]

    # The attribution text
    attribution: Var[str]

    # The map pane where the circle marker will be added
    pane: Var[str]


class Polyline(BaseLeafletComponent):
    """Polyline component for displaying a line on a map."""

    tag = "Polyline"

    # Array of points making up the polyline
    positions: Var[List[LatLng]]

    # Styling options for the polyline
    path_options: Var[PathOptions]

    # The attribution text
    attribution: Var[str]

    # The map pane where the polyline will be added
    pane: Var[str]


class Polygon(BaseLeafletComponent):
    """Polygon component for displaying a filled polygon on a map."""

    tag = "Polygon"

    # Array of points making up the polygon
    positions: Var[List[LatLng]]

    # Styling options for the polygon
    path_options: Var[PathOptions]

    # The attribution text
    attribution: Var[str]

    # The map pane where the polygon will be added
    pane: Var[str]


class Rectangle(BaseLeafletComponent):
    """Rectangle component for displaying a rectangle on a map."""

    tag = "Rectangle"

    # The bounds of the rectangle
    bounds: Var[LatLngBounds]

    # Styling options for the rectangle
    path_options: Var[PathOptions]

    # The attribution text
    attribution: Var[str]

    # The map pane where the rectangle will be added
    pane: Var[str]


class SVGOverlay(BaseLeafletComponent):
    """SVGOverlay component for displaying custom SVG on a map."""

    tag = "SVGOverlay"

    # The bounds where the SVG will be drawn
    bounds: Var[LatLngBounds]

    # SVG attributes
    attributes: Var[Dict[str, str]]

    # The attribution text
    attribution: Var[str]

    # The map pane where the SVG overlay will be added
    pane: Var[str]
